@@ -12,41 +12,41 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Optional
-
-import jax
-import jax.numpy as jnp
 
 from textwrap import dedent
-from inspect import signature
 
-from netket.utils.types import PyTree
+from netket.utils import timing
+from netket.utils.types import PyTree, Optimizer
 from netket.operator import AbstractOperator
 from netket.stats import Stats
-from netket.vqs import MCState
 from netket.optimizer import (
     identity_preconditioner,
     PreconditionerT,
-    _DeprecatedPreconditionerSignature,
 )
+from netket.vqs import VariationalState
+from netket.jax import tree_cast
 
-from .vmc_common import info
 from .abstract_variational_driver import AbstractVariationalDriver
 
 
 class VMC(AbstractVariationalDriver):
     """
     Energy minimization using Variational Monte Carlo (VMC).
+
+    .. note::
+
+        Consider using directly the new :class:``netket.experimental.driver.VMC_SR``
+        class instead of this one if you want to use Stochastic Reconfiguration.
+
     """
 
     def __init__(
         self,
         hamiltonian: AbstractOperator,
-        optimizer,
-        *args,
-        variational_state=None,
+        optimizer: Optimizer,
+        *,
+        variational_state: VariationalState,
         preconditioner: PreconditionerT = identity_preconditioner,
-        **kwargs,
     ):
         """
         Initializes the driver class.
@@ -55,15 +55,14 @@ class VMC(AbstractVariationalDriver):
             hamiltonian: The Hamiltonian of the system.
             optimizer: Determines how optimization steps are performed given the
                 bare energy gradient.
+            variational_state: The variational state for which the hamiltonian must
+                be minimised.
             preconditioner: Determines which preconditioner to use for the loss gradient.
                 This must be a tuple of `(object, solver)` as documented in the section
                 `preconditioners` in the documentation. The standard preconditioner
                 included with NetKet is Stochastic Reconfiguration. By default, no
                 preconditioner is used and the bare gradient is passed to the optimizer.
         """
-        if variational_state is None:
-            variational_state = MCState(*args, **kwargs)
-
         if variational_state.hilbert != hamiltonian.hilbert:
             raise TypeError(
                 dedent(
@@ -83,7 +82,6 @@ class VMC(AbstractVariationalDriver):
 
         self._dp: PyTree = None
         self._S = None
-        self._sr_info = None
 
     @property
     def preconditioner(self):
@@ -103,21 +101,19 @@ class VMC(AbstractVariationalDriver):
         argument is the step, used to change some parameters along the
         optimisation.
 
-        Often, this is taken to be :func:`nk.optimizer.SR`. If it is set to
-        `None`, then the identity is used.
+        Often, this is taken to be :func:`~netket.optimizer.SR`. If it is
+        set to `None`, then the identity is used.
         """
         return self._preconditioner
 
     @preconditioner.setter
-    def preconditioner(self, val: Optional[PreconditionerT]):
+    def preconditioner(self, val: PreconditionerT | None):
         if val is None:
             val = identity_preconditioner
 
-        if len(signature(val).parameters) == 2:
-            val = _DeprecatedPreconditionerSignature(val)
-
         self._preconditioner = val
 
+    @timing.timed
     def _forward_and_backward(self):
         """
         Performs a number of VMC optimization steps.
@@ -136,11 +132,7 @@ class VMC(AbstractVariationalDriver):
         self._dp = self.preconditioner(self.state, self._loss_grad, self.step_count)
 
         # If parameters are real, then take only real part of the gradient (if it's complex)
-        self._dp = jax.tree_map(
-            lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
-            self._dp,
-            self.state.parameters,
-        )
+        self._dp = tree_cast(self._dp, self.state.parameters)
 
         return self._dp
 
@@ -158,15 +150,3 @@ class VMC(AbstractVariationalDriver):
             + f"\n  step_count = {self.step_count},"
             + f"\n  state = {self.state})"
         )
-
-    def info(self, depth=0):
-        lines = [
-            "{}: {}".format(name, info(obj, depth=depth + 1))
-            for name, obj in [
-                ("Hamiltonian    ", self._ham),
-                ("Optimizer      ", self._optimizer),
-                ("Preconditioner ", self.preconditioner),
-                ("State          ", self.state),
-            ]
-        ]
-        return "\n{}".format(" " * 3 * (depth + 1)).join([str(self)] + lines)
