@@ -15,14 +15,19 @@
 __all__ = [
     "driver",
     "dynamics",
+    "geometry",
     "sampler",
     "vqs",
     "TDVP",
+    "models",
     "hilbert",
     "operator",
     "logging",
+    "observable",
+    "QSR",
 ]
 
+from . import geometry
 from . import hilbert
 from . import operator
 from . import driver
@@ -31,6 +36,8 @@ from . import sampler
 from . import vqs
 from . import logging
 from . import qsr
+from . import observable
+from . import models
 
 from .driver import TDVP
 from .qsr import QSR
