@@ -12,20 +12,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable, Union
+from collections.abc import Callable
 from functools import partial
 
 import jax
-import jax.numpy as jnp
 
 import netket as nk
-from netket.driver.vmc_common import info
+from netket.jax import tree_cast
 from netket.operator import AbstractOperator
 from netket.optimizer import LinearOperator
 from netket.optimizer.qgt import QGTAuto
-from netket.vqs import VariationalState, VariationalMixedState, MCState, ExactState
-
-from netket.experimental.dynamics import RKIntegratorConfig
+from netket.vqs import (
+    VariationalState,
+    VariationalMixedState,
+    MCState,
+    FullSumState,
+)
+from netket.experimental.dynamics import AbstractSolver
 
 from .tdvp_common import TDVPBaseDriver, odefun
 
@@ -35,29 +38,20 @@ class TDVP(TDVPBaseDriver):
     Variational time evolution based on the time-dependent variational principle which,
     when used with Monte Carlo sampling via :class:`netket.vqs.MCState`, is the time-dependent VMC
     (t-VMC) method.
-
-    .. note::
-        This TDVP Driver uses the time-integrators from the `nkx.dynamics` module, which are
-        automatically executed under a `jax.jit` context.
-
-        When running computations on GPU, this can lead to infinite hangs or extremely long
-        compilation times. In those cases, you might try setting the configuration variable
-        `nk.config.netket_experimental_disable_ode_jit = True` to mitigate those issues.
-
     """
 
     def __init__(
         self,
         operator: AbstractOperator,
         variational_state: VariationalState,
-        integrator: RKIntegratorConfig,
+        ode_solver: AbstractSolver,
         *,
         t0: float = 0.0,
-        propagation_type="real",
-        qgt: LinearOperator = None,
-        linear_solver=nk.optimizer.solver.svd,
+        propagation_type: str = "real",
+        qgt: LinearOperator | None = None,
+        linear_solver=nk.optimizer.solver.pinv_smooth,
         linear_solver_restart: bool = False,
-        error_norm: Union[str, Callable] = "euclidean",
+        error_norm: str | Callable = "euclidean",
     ):
         r"""
         Initializes the time evolution driver.
@@ -66,7 +60,7 @@ class TDVP(TDVPBaseDriver):
             operator: The generator of the dynamics (Hamiltonian for pure states,
                 Lindbladian for density operators).
             variational_state: The variational state.
-            integrator: Configuration of the algorithm used for solving the ODE.
+            ode_solver: Solving algorithm used the ODE.
             t0: Initial time at the start of the time evolution.
             propagation_type: Determines the equation of motion: "real"  for the
                 real-time Schrödinger equation (SE), "imag" for the imaginary-time SE.
@@ -75,9 +69,9 @@ class TDVP(TDVPBaseDriver):
                 This must be a jax-jittable function :code:`f(A,b) -> x` that accepts a Matrix-like, Linear Operator
                 PyTree object :math:`A` and a vector-like PyTree :math:`b` and returns the PyTree :math:`x` solving
                 the system :math:`Ax=b`.
-                Defaults to :func:`nk.optimizer.solver.svd` with the default svd threshold of 1e-10.
+                Defaults to :func:`nk.optimizer.solver.pinv_smooth` with the default svd threshold of 1e-10.
                 To change the svd threshold you can use :func:`functools.partial` as follows:
-                :code:`functools.partial(nk.optimizer.solver.svd, rcond=1e-4)`.
+                :code:`functools.partial(nk.optimizer.solver.pinv_smooth, rcond_smooth=1e-5)`.
             linear_solver_restart: If False (default), the last solution of the linear system
                 is used as initial value in subsequent steps.
             error_norm: Norm function used to calculate the error with adaptive integrators.
@@ -117,25 +111,17 @@ class TDVP(TDVPBaseDriver):
         self.linear_solver_restart = linear_solver_restart
 
         super().__init__(
-            operator, variational_state, integrator, t0=t0, error_norm=error_norm
+            operator,
+            variational_state,
+            ode_solver,
+            t0=t0,
+            error_norm=error_norm,
         )
 
-    def info(self, depth=0):
-        lines = [
-            "{}: {}".format(name, info(obj, depth=depth + 1))
-            for name, obj in [
-                ("generator     ", self._generator_repr),
-                ("integrator    ", self._integrator),
-                ("linear solver ", self.linear_solver),
-                ("state         ", self.state),
-            ]
-        ]
-        return "\n{}".format(" " * 3 * (depth + 1)).join([str(self)] + lines)
-
 
 @odefun.dispatch
 def odefun_tdvp(  # noqa: F811
-    state: Union[MCState, ExactState], driver: TDVP, t, w, *, stage=0
+    state: MCState | FullSumState, driver: TDVP, t, w, *, stage=0
 ):
     # pylint: disable=protected-access
 
@@ -163,28 +149,19 @@ def odefun_tdvp(  # noqa: F811
     driver._dw, _ = qgt.solve(driver.linear_solver, driver._loss_grad, x0=initial_dw)
 
     # If parameters are real, then take only real part of the gradient (if it's complex)
-    driver._dw = jax.tree_map(
-        lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
-        driver._dw,
-        state.parameters,
-    )
+    driver._dw = tree_cast(driver._dw, state.parameters)
 
     return driver._dw
 
 
 @partial(jax.jit, static_argnums=(3, 4))
 def _map_parameters(forces, parameters, loss_grad_factor, propagation_type, state_T):
-
-    forces = jax.tree_map(
+    forces = jax.tree_util.tree_map(
         lambda x, target: loss_grad_factor * x,
         forces,
         parameters,
     )
 
-    forces = jax.tree_map(
-        lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
-        forces,
-        parameters,
-    )
+    forces = tree_cast(forces, parameters)
 
     return forces
