@@ -14,8 +14,10 @@
 
 
 __all__ = [
-    "RungeKuttaIntegrator",
-    "RKIntegratorConfig",
+    "AbstractSolver",
+    "AbstractSolverState",
+    "IntegratorState",
+    "Integrator",
     "Euler",
     "Heun",
     "Midpoint",
@@ -25,8 +27,10 @@ __all__ = [
     "RK45",
 ]
 
-from ._rk_solver_structures import RungeKuttaIntegrator, RKIntegratorConfig
-from ._rk_solver import Euler, Heun, Midpoint, RK4, RK12, RK23, RK45
+from ._solver import AbstractSolver, AbstractSolverState
+from ._integrator_state import IntegratorState
+from ._integrator import Integrator
+from ._rk._solver import Euler, Heun, Midpoint, RK4, RK12, RK23, RK45
 
 from netket.utils import _hide_submodules
 
