@@ -15,6 +15,10 @@
 import os
 import numpy as np
 from flax.serialization import to_bytes
+from flax.core import pop as fpop, FrozenDict
+
+from netket.logging import AbstractLog
+from netket.jax.sharding import extract_replicated
 
 _mode_shorthands = {"write": "w", "append": "a", "fail": "x"}
 
@@ -35,7 +39,7 @@ def tree_log(tree, root, data, *, iter=None):
         return
 
     elif isinstance(tree, list):
-        for (i, val) in enumerate(tree):
+        for i, val in enumerate(tree):
             tree_log(val, f"{root}/{i}", data, iter=iter)
 
     # handle namedtuples
@@ -46,7 +50,7 @@ def tree_log(tree, root, data, *, iter=None):
 
     elif isinstance(tree, tuple):
         tree_log(iter, f"{root}/iter", data)
-        for (i, val) in enumerate(tree):
+        for i, val in enumerate(tree):
             tree_log(val, f"{root}/{i}", data)
 
     elif isinstance(tree, dict):
@@ -71,11 +75,11 @@ def tree_log(tree, root, data, *, iter=None):
             f_value.resize(f_value.shape[0] + 1, axis=0)
             f_value[-1] = value
         else:
-            maxshape = (None,) + value.shape
+            maxshape = (None, *value.shape)
             data.create_dataset(root, data=[value], maxshape=maxshape)
 
 
-class HDF5Log:
+class HDF5Log(AbstractLog):
     r"""
     HDF5 Logger, that can be passed with keyword argument `logger` to Monte
     Carlo drivers in order to serialize the output data of the simulation.
@@ -160,25 +164,36 @@ class HDF5Log:
         self._steps_notsaved_params = 0
 
     def _init_output_file(self):
-        import h5py
+        if self._is_master_process:
+            import h5py
 
-        self._writer = h5py.File(self._file_name, self._file_mode)
+            self._writer = h5py.File(self._file_name, self._file_mode)
 
     def __call__(self, step, log_data, variational_state):
         if self._writer is None:
             self._init_output_file()
 
-        tree_log(log_data, "data", self._writer, iter=step)
-
         if self._steps_notsaved_params % self._save_params_every == 0:
-            variables = variational_state.variables.unfreeze()
-            params = variables.pop("params")
-            binary_data = to_bytes(variables)
-            tree = {"model_state": binary_data, "parameters": params, "iter": step}
-            tree_log(tree, "variational_state", self._writer)
+            variables = variational_state.variables
+
+            # TODO: remove - FrozenDict are deprecated
+            if isinstance(variables, FrozenDict):
+                variables = variables.unfreeze()
+            variables = extract_replicated(variables)
             self._steps_notsaved_params = 0
+        else:
+            variables = None
+
+        if self._is_master_process:
+            tree_log(log_data, "data", self._writer, iter=step)
+
+            if variables is not None:
+                _, params = fpop(variables, "params")
+                binary_data = to_bytes(variables)
+                tree = {"model_state": binary_data, "parameters": params, "iter": step}
+                tree_log(tree, "variational_state", self._writer)
+            self._writer.flush()
 
-        self._writer.flush()
         self._steps_notsaved_params += 1
 
     def flush(self, variational_state=None):
@@ -191,6 +206,10 @@ class HDF5Log:
         if self._writer is not None:
             self._writer.flush()
 
+    def __del__(self):
+        if hasattr(self, "_writer"):
+            self.flush()
+
     def __repr__(self):
         _str = f"HDF5Log('{self._file_name}', mode={self._file_mode}"
         return _str
