@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import List, Optional, Tuple, Union
+from typing import Union
 import warnings
 
 import numpy as np
@@ -21,7 +21,6 @@ import jax.numpy as jnp
 
 from netket import jax as nkjax
 from netket.driver import AbstractVariationalDriver
-from netket.driver.vmc_common import info
 from netket.operator import AbstractOperator
 from netket.vqs import VariationalState
 from netket.optimizer import (
@@ -29,6 +28,8 @@ from netket.optimizer import (
     PreconditionerT,
 )
 from netket.utils.types import Array
+from netket.jax import tree_cast
+
 
 from netket.stats import statistics
 
@@ -95,21 +96,16 @@ class QSR(AbstractVariationalDriver):
 
     def __init__(
         self,
-        training_data: Union[RawQuantumDataset, Tuple[List, List]],
+        training_data: RawQuantumDataset | tuple[list, list],
         training_batch_size: int,
         optimizer,
         *,
         variational_state: VariationalState,
-        preconditioner: Optional[PreconditionerT] = identity_preconditioner,
-        seed: Optional[int] = None,
-        batch_sample_replace: Optional[bool] = True,
-        control_variate_update_freq: Optional[
-            Union[
-                int,
-                str,
-            ]
-        ] = None,
-        chunk_size: Optional[int] = None,
+        preconditioner: PreconditionerT | None = identity_preconditioner,
+        seed: int | None = None,
+        batch_sample_replace: bool | None = True,
+        control_variate_update_freq: None | (int | str) = None,
+        chunk_size: int | None = None,
     ):
         r"""Initializes the QSR driver class.
 
@@ -138,9 +134,7 @@ class QSR(AbstractVariationalDriver):
         if not isinstance(training_data, RawQuantumDataset):
             training_data = RawQuantumDataset(training_data)
 
-        self._rng = np.random.default_rng(
-            np.asarray(nkjax.mpi_split(nkjax.PRNGKey(seed)))
-        )
+        self._rng = np.random.default_rng(np.asarray(nkjax.PRNGKey(seed)))
 
         # mixed states
         self.mixed_states = variational_state.__class__.__name__ in ["MCMixedState"]
@@ -175,7 +169,8 @@ class QSR(AbstractVariationalDriver):
             self.n_chunk = self.dataset.size // self._chunk_size
             if not self.n_chunk * self._chunk_size == self.dataset.size:
                 warnings.warn(
-                    "WARNING: chunk size does not divide the number of samples, the last few chunks will be smaller"
+                    "WARNING: chunk size does not divide the number of samples, the last few chunks will be smaller",
+                    stacklevel=2,
                 )
             self._chunked_indices = np.array_split(
                 np.arange(self.dataset.size), self.n_chunk
@@ -266,7 +261,7 @@ class QSR(AbstractVariationalDriver):
 
             # gather gradient
             # grad <- grad - grad_cv + E[grad_cv]
-            self._grad_pos = jax.tree_map(
+            self._grad_pos = jax.tree_util.tree_map(
                 lambda x, y, Ey: x - y + Ey,
                 self._grad_pos,
                 self._grad_pos_cv,
@@ -279,36 +274,28 @@ class QSR(AbstractVariationalDriver):
 
         # restore the square in prob = |psi|^2
         if not self.mixed_states:
-            self._loss_grad = jax.tree_map(lambda x: x * 2.0, self._loss_grad)
+            self._loss_grad = jax.tree_util.tree_map(lambda x: x * 2.0, self._loss_grad)
 
         # If parameters are real, then take only real part of the gradient (if it's complex)
-        self._loss_grad = jax.tree_map(
-            lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
-            self._loss_grad,
-            self.state.parameters,
-        )
+        self._loss_grad = tree_cast(self._loss_grad, self.state.parameters)
 
         # if it's the identity it does
         # self._dp = self._loss_grad
         self._dp = self.preconditioner(self.state, self._loss_grad)
 
         # If parameters are real, then take only real part of the gradient (if it's complex)
-        self._dp = jax.tree_map(
-            lambda x, target: (x if jnp.iscomplexobj(target) else x.real),
-            self._dp,
-            self.state.parameters,
-        )
+        self._dp = tree_cast(self._dp, self.state.parameters)
 
         return self._dp
 
-    def nll(self, return_stats: Optional[bool] = True):
+    def nll(self, return_stats: bool | None = True):
         r"""
         Compute the Negative-Log-Likelihood over a batch of data.
 
         Args:
             return_stats: if True, return the statistics.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
 
@@ -341,14 +328,14 @@ class QSR(AbstractVariationalDriver):
             return statistics(jnp.real(log_n - ce))
         return jnp.real(log_n - ce)
 
-    def nll_whole_training_set(self, return_stats: Optional[bool] = True):
+    def nll_whole_training_set(self, return_stats: bool | None = True):
         r"""
         Compute the Negative-Log-Likelihood over the whole training set.
 
         Args:
             return_stats: if True, return the statistics.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
         """
@@ -400,8 +387,8 @@ class QSR(AbstractVariationalDriver):
     def entropy(
         self,
         target_state: Array,
-        n_shots: Optional[int] = 1,
-        no_cache: Optional[bool] = False,
+        n_shots: int | None = 1,
+        no_cache: bool | None = False,
     ) -> float:
         r"""
         Compute the average entropy of the probability distributions
@@ -412,7 +399,7 @@ class QSR(AbstractVariationalDriver):
             n_shots: number of shots per measurement basis.
             no_cache: if True, do not use the cached value.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
 
@@ -436,7 +423,7 @@ class QSR(AbstractVariationalDriver):
         self._entropy = np.mean(entropy_list)
         return self._entropy
 
-    def KL(self, target_state: Optional[Array] = None, n_shots: Optional[int] = None):
+    def KL(self, target_state: Array | None = None, n_shots: int | None = None):
         r"""
         Compute average KL divergence loss over a batch of data.
 
@@ -444,7 +431,7 @@ class QSR(AbstractVariationalDriver):
             target_state: the target state.
             n_shots: number of shots per measurement basis.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
 
@@ -456,7 +443,7 @@ class QSR(AbstractVariationalDriver):
         )
 
     def KL_whole_training_set(
-        self, target_state: Optional[Array] = None, n_shots: Optional[int] = None
+        self, target_state: Array | None = None, n_shots: int | None = None
     ):
         r"""
         Compute average KL divergence loss over the whole training set.
@@ -465,7 +452,7 @@ class QSR(AbstractVariationalDriver):
             target_state: the target state.
             n_shots: number of shots per measurement basis.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
 
@@ -478,7 +465,7 @@ class QSR(AbstractVariationalDriver):
         )
 
     def KL_exact(
-        self, target_state: Optional[Array] = None, n_shots: Optional[int] = 1
+        self, target_state: Array | None = None, n_shots: int | None = 1
     ) -> float:
         r"""
         Compute the average KL divergence loss between the variational state and the target state.
@@ -487,7 +474,7 @@ class QSR(AbstractVariationalDriver):
             target_state: the target state.
             n_shots: number of shots per measurement basis.
 
-        .. warn::
+        .. warning::
 
             Exponentially expensive in the hilbert space size!
 
@@ -519,14 +506,3 @@ class QSR(AbstractVariationalDriver):
             + f"\n  step_count = {self.step_count},"
             + f"\n  state = {self.state})"
         )
-
-    def info(self, depth=0):
-        lines = [
-            "{}: {}".format(name, info(obj, depth=depth + 1))
-            for name, obj in [
-                ("Optimizer   ", self._optimizer),
-                ("SR solver   ", self.sr),
-                ("State       ", self.state),
-            ]
-        ]
-        return "\n{}".format(" " * 3 * (depth + 1)).join([str(self)] + lines)
