@@ -13,11 +13,12 @@
 # limitations under the License.
 
 import abc
-from typing import List, Iterator, Sequence, Tuple, Union, Optional
+from typing import Union
+from collections.abc import Sequence
 
 
-Edge = Tuple[int, int]
-ColoredEdge = Tuple[int, int, int]
+Edge = tuple[int, int]
+ColoredEdge = tuple[int, int, int]
 EdgeSequence = Union[Sequence[Edge], Sequence[ColoredEdge]]
 
 
@@ -36,7 +37,7 @@ class AbstractGraph(abc.ABC):
 
     @abc.abstractmethod
     def edges(
-        self, return_color: bool = False, filter_color: Optional[int] = None
+        self, return_color: bool = False, filter_color: int | None = None
     ) -> EdgeSequence:
         r"""Returns the sequence of edges of the graph.
 
@@ -59,12 +60,12 @@ class AbstractGraph(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def nodes(self) -> Iterator[int]:
+    def nodes(self) -> Sequence[int]:
         r"""Iterator over the nodes of the graph"""
         raise NotImplementedError
 
     @abc.abstractmethod
-    def distances(self) -> List[List]:
+    def distances(self) -> list[list]:
         r"""List containing the distances between the nodes.
         The distance between unconnected nodes (no path exists between them) is set to -1
         """
@@ -87,7 +88,7 @@ class AbstractGraph(abc.ABC):
         return len(self.edges())
 
     @abc.abstractmethod
-    def adjacency_list(self) -> List[List]:
+    def adjacency_list(self) -> list[list]:
         r"""List containing the adjacency list of the graph where each node
         is represented by an integer in [0, n_nodes)"""
         raise NotImplementedError
