@@ -12,21 +12,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import warnings
+
+from . import constraint
+from . import index
+
 from .abstract_hilbert import AbstractHilbert
 from .discrete_hilbert import DiscreteHilbert
 from .homogeneous import HomogeneousHilbert
 
-from .continuous_hilbert import ContinuousHilbert
 
-from .custom_hilbert import CustomHilbert
 from .doubled_hilbert import DoubledHilbert
 from .spin import Spin
 from .fock import Fock
 from .qubit import Qubit
-from .particle import Particle
+from .spin_orbital_fermions import SpinOrbitalFermions
 
 from .tensor_hilbert import TensorHilbert
+from . import tensor_hilbert_discrete
 
+from . import random
 
 from netket.utils import _hide_submodules
 
