@@ -12,13 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import List, Optional, Union
-from functools import partial
-
+from collections.abc import Sequence
+from typing import Self
 import numpy as np
-from numba import jit
+
+from netket.utils import StaticRange
 
 from .homogeneous import HomogeneousHilbert
+from .constraint import DiscreteHilbertConstraint, SumConstraint
 
 FOCK_MAX = np.iinfo(np.intp).max - 1
 """
@@ -28,19 +29,15 @@ and it would overflow.
 """
 
 
-@jit(nopython=True)
-def _sum_constraint(x, n_particles):
-    return np.sum(x, axis=1) == n_particles
-
-
 class Fock(HomogeneousHilbert):
     r"""Hilbert space obtained as tensor product of local fock basis."""
 
     def __init__(
         self,
-        n_max: Optional[int] = None,
+        n_max: int | None = None,
         N: int = 1,
-        n_particles: Optional[int] = None,
+        n_particles: int | None = None,
+        constraint: DiscreteHilbertConstraint | None = None,
     ):
         r"""
         Constructs a new ``Boson`` given a maximum occupation number, number of sites
@@ -52,6 +49,9 @@ class Fock(HomogeneousHilbert):
           N: number of bosonic modes (default = 1)
           n_particles: Constraint for the number of particles. If None, no constraint
             is imposed.
+          constraint: A custom constraint on the allowed configurations. This argument
+            cannot be specified at the same time as :code:`n_particles`. The constraint
+            must be a subclass of :class:`~netket.hilbert.constraint.DiscreteHilbertConstraint`
 
         Examples:
            Simple boson hilbert space.
@@ -66,12 +66,22 @@ class Fock(HomogeneousHilbert):
         self._n_max = n_max
 
         if n_particles is not None:
+            if not isinstance(n_particles, int):
+                raise TypeError(
+                    f"n_particles must be an integer. Got {n_particles} ({type(n_particles)})"
+                )
+            if constraint is not None:
+                raise ValueError(
+                    "Cannot specify at the same time a total magnetization "
+                    "constraint and a `custom_constraint."
+                )
+
             n_particles = int(n_particles)
             if n_particles < 0:
                 raise ValueError(
                     f"Number of particles must be >= 0, but received {n_particles}."
                 )
-            self._n_particles = n_particles
+            self._n_particles: int | None = n_particles
 
             if self._n_max is None:
                 self._n_max = n_particles
@@ -81,49 +91,47 @@ class Fock(HomogeneousHilbert):
                         """The required total number of bosons is not compatible
                         with the given n_max."""
                     )
-
-            constraints = partial(_sum_constraint, n_particles=n_particles)
-
+            constraint = SumConstraint(n_particles)
         else:
-            constraints = None
             self._n_particles = None
 
-        if self._n_max is not None:
-            # assert self._n_max > 0
-            local_states = np.arange(self._n_max + 1)
-        else:
+        if self._n_max is None:
             self._n_max = FOCK_MAX
-            local_states = None
+        local_states = StaticRange(
+            0,
+            1,
+            self._n_max + 1,
+        )
 
-        super().__init__(local_states, N, constraints)
+        super().__init__(local_states, N, constraint=constraint)
 
     @property
-    def n_max(self) -> Optional[int]:
+    def n_max(self) -> int | None:
         r"""The maximum number of bosons per site, or None
         if the number is unconstrained."""
         return self._n_max
 
     @property
-    def n_particles(self) -> Optional[int]:
+    def n_particles(self) -> int | None:
         r"""The total number of particles, or None
         if the number is unconstrained."""
         return self._n_particles
 
     def __pow__(self, n) -> "Fock":
-        if self.n_particles is None:
+        if not self.constrained:
             return Fock(self.n_max, self.size * n)
 
         return NotImplemented
 
-    def _mul_sametype_(self, other: "Fock") -> "Fock":
+    def _mul_sametype_(self, other: Self) -> Self:
         assert type(self) == type(other)
         if self.n_max == other.n_max:
-            if self._n_particles is None and other._n_particles is None:
+            if not self.constrained and not other.constrained:
                 return Fock(self.n_max, N=self.size + other.size)
 
         return NotImplemented
 
-    def ptrace(self, sites: Union[int, List]) -> Optional["Fock"]:
+    def ptrace(self: Self, sites: int | Sequence[int]) -> Self | None:
         if isinstance(sites, int):
             sites = [sites]
 
@@ -133,7 +141,7 @@ class Fock(HomogeneousHilbert):
                     f"Site {site} not in this hilbert space of site {self.size}"
                 )
 
-        if self.n_particles is not None:
+        if self.constrained:
             raise TypeError(
                 "Cannot take the partial trace with a total particles constraint."
             )
@@ -146,17 +154,16 @@ class Fock(HomogeneousHilbert):
             return Fock(self.n_max, N=self.size - Nsites)
 
     def __repr__(self):
-        n_particles = (
-            ", n_particles={}".format(self._n_particles)
-            if self._n_particles is not None
-            else ""
-        )
-        nmax = self._n_max if self._n_max < FOCK_MAX else "FOCK_MAX"
-        return "Fock(n_max={}{}, N={})".format(nmax, n_particles, self.size)
+        if self.n_particles is not None:
+            constraint = f", n_particles={self.n_particles}"
+        elif self.constrained:
+            constraint = f", {self._constraint}"
+        else:
+            constraint = ""
 
-    def states_to_local_indices(self, x):
-        return x.astype(np.int32)
+        nmax = self._n_max if self._n_max < FOCK_MAX else "FOCK_MAX"
+        return f"Fock(n_max={nmax}{constraint}, N={self.size})"
 
     @property
     def _attrs(self):
-        return (self.size, self._n_max, self._n_particles)
+        return (self.size, self._n_max, self.constraint)
