@@ -14,41 +14,58 @@
 
 from netket.utils import HashablePartial
 
-from .utils import (
-    tree_ravel,
+from ._utils_dtype import (
     is_complex_dtype,
-    tree_size,
-    eval_shape,
-    tree_leaf_iscomplex,
-    tree_ishomogeneous,
     dtype_complex,
     dtype_real,
     maybe_promote_to_complex,
+    canonicalize_dtypes,
+)
+
+from ._utils_tree import (
+    tree_ravel,
+    tree_size,
+    eval_shape,
+    tree_leaf_isreal,
+    tree_leaf_iscomplex,
+    tree_ishomogeneous,
     tree_conj,
     tree_dot,
+    tree_norm,
     tree_cast,
+    tree_ax,
     tree_axpy,
     tree_to_real,
     compose,
-    mpi_split,
+)
+
+from ._utils_random import (
     PRNGKey,
     PRNGSeq,
+    batch_choice,
 )
 
+
 from ._vjp import vjp
 from ._grad import grad, value_and_grad
 
-from ._expect import expect
-
 from ._chunk_utils import chunk, unchunk
 from ._scanmap import scan_reduce, scan_append, scan_append_reduce, scanmap
 from ._vjp_chunked import vjp_chunked
 from ._vmap_chunked import apply_chunked, vmap_chunked
 
-from ._math import logsumexp_cplx
+from ._math import logsumexp_cplx, logdet_cmplx
 
-from ._jacobian import jacobian
+from ._jacobian import jacobian, jacobian_default_mode
+
+from ._sparse import COOArray
+from ._sort import sort, searchsorted
+
+from ._expect import expect
+
+# internal sharding utilities
+from . import sharding
 
 from netket.utils import _hide_submodules
 
-_hide_submodules(__name__)
+_hide_submodules(__name__, ignore="sharding")
