@@ -12,19 +12,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable, Tuple, Any, Union, Sequence
+from typing import Any
+from collections.abc import Callable
+from collections.abc import Sequence
 import operator
 
 import jax
 import jax.numpy as jnp
 from jax.util import safe_map
 
-from .utils import tree_leaf_iscomplex, eval_shape
+from ._utils_tree import tree_leaf_iscomplex, eval_shape
 
 map = safe_map
 
 
-def _ensure_index(x: Any) -> Union[int, Tuple[int, ...]]:
+def _ensure_index(x: Any) -> int | tuple[int, ...]:
     """Ensure x is either an index or a tuple of indices."""
     try:
         return operator.index(x)
@@ -34,7 +36,7 @@ def _ensure_index(x: Any) -> Union[int, Tuple[int, ...]]:
 
 def grad(
     fun: Callable,
-    argnums: Union[int, Sequence[int]] = 0,
+    argnums: int | Sequence[int] = 0,
     has_aux: bool = False,
     allow_int: bool = False,
 ) -> Callable:
@@ -91,10 +93,10 @@ def grad(
 
 def value_and_grad(
     fun: Callable,
-    argnums: Union[int, Sequence[int]] = 0,
+    argnums: int | Sequence[int] = 0,
     has_aux: bool = False,
     allow_int: bool = False,
-) -> Callable[..., Tuple[Any, Any]]:
+) -> Callable[..., tuple[Any, Any]]:
     """Create a function which evaluates both ``fun`` and the gradient of ``fun``.
 
     Args:
@@ -187,7 +189,9 @@ def value_and_grad(
                         )(*args, **kwargs)
 
                     out = out_r + 1j * out_j
-                    grad = jax.tree_map(lambda re, im: re + 1j * im, grad_r, grad_j)
+                    grad = jax.tree_util.tree_map(
+                        lambda re, im: re + 1j * im, grad_r, grad_j
+                    )
 
                     if has_aux:
                         return out, grad, aux
