@@ -18,8 +18,6 @@ from functools import wraps
 import jax
 from jax import numpy as jnp
 
-from netket.utils import deprecated
-
 from jax.nn import celu
 from jax.nn import elu
 from jax.nn import gelu
@@ -28,7 +26,7 @@ from jax.nn import leaky_relu
 
 from jax.nn import log_sigmoid
 from jax.nn import log_softmax
-from jax.nn import normalize
+from jax.nn import standardize
 from jax.nn import relu
 
 from jax.nn import sigmoid
@@ -48,6 +46,7 @@ from jax.numpy import tanh
 from jax.numpy import cosh
 from jax.numpy import sinh
 
+
 from netket.jax import HashablePartial
 
 
@@ -64,7 +63,7 @@ def reim(f):
 
     fun = HashablePartial(reim_f, f)
 
-    fun.__name__ = f"reim_{f.__name__}"
+    fun.__name__ = f"reim_{f.__name__}"  # type: ignore
     fun.__doc__ = (
         f"{f.__name__} applied separately to the real and"
         f"imaginary parts of it's input.\n\n"
