@@ -12,18 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Tuple
+from typing import Any
 
 import numpy as np
 from flax import linen as nn
 from flax.linen.dtypes import promote_dtype
 
+import jax
 from jax import lax
 from jax import numpy as jnp
 from jax.nn.initializers import lecun_normal, zeros
 
 from netket.utils.types import Array, DType, NNInitFunc
-from netket.utils import deprecate_dtype
 
 default_kernel_init = lecun_normal()
 
@@ -43,13 +43,12 @@ def wrap_kernel_init(kernel_init, mask):
 def _conv_dimension_numbers(input_shape):
     """Computes the dimension numbers based on the input shape."""
     ndim = len(input_shape)
-    lhs_spec = (0, ndim - 1) + tuple(range(1, ndim - 1))
-    rhs_spec = (ndim - 1, ndim - 2) + tuple(range(0, ndim - 2))
+    lhs_spec = (0, ndim - 1, *tuple(range(1, ndim - 1)))
+    rhs_spec = (ndim - 1, ndim - 2, *tuple(range(0, ndim - 2)))
     out_spec = lhs_spec
     return lax.ConvDimensionNumbers(lhs_spec, rhs_spec, out_spec)
 
 
-@deprecate_dtype
 class MaskedDense1D(nn.Module):
     """1D linear transformation module with mask for autoregressive NN."""
 
@@ -69,7 +68,7 @@ class MaskedDense1D(nn.Module):
     """initializer for the bias."""
 
     @nn.compact
-    def __call__(self, inputs: Array) -> Array:
+    def __call__(self, inputs: jax.Array) -> Array:
         """
         Applies a masked linear transformation to the inputs.
 
@@ -100,7 +99,7 @@ class MaskedDense1D(nn.Module):
         mask = np.kron(
             mask, np.ones((in_features, self.features), dtype=self.param_dtype)
         )
-        mask = jnp.asarray(mask)
+        mask = jnp.asarray(mask)  # type: ignore[no-redef]
 
         kernel = self.param(
             "kernel",
@@ -126,7 +125,6 @@ class MaskedDense1D(nn.Module):
         return y
 
 
-@deprecate_dtype
 class MaskedConv1D(nn.Module):
     """1D convolution module with mask for autoregressive NN."""
 
@@ -208,7 +206,7 @@ class MaskedConv1D(nn.Module):
         dimension_numbers = _conv_dimension_numbers(inputs.shape)
         y = lax.conv_general_dilated(
             y,
-            kernel,
+            kernel,  # type: ignore[arg-type]
             window_strides=(1,),
             padding="VALID",
             lhs_dilation=(1,),
@@ -227,15 +225,14 @@ class MaskedConv1D(nn.Module):
         return y
 
 
-@deprecate_dtype
 class MaskedConv2D(nn.Module):
     """2D convolution module with mask for autoregressive NN."""
 
     features: int
     """number of convolution filters."""
-    kernel_size: Tuple[int, int]
+    kernel_size: tuple[int, int]
     """shape of the convolutional kernel `(h, w)`. Typically, `h = w // 2 + 1`."""
-    kernel_dilation: Tuple[int, int]
+    kernel_dilation: tuple[int, int]
     """a sequence of 2 integers, giving the dilation factor to
     apply in each spatial dimension of the convolution kernel."""
     exclusive: bool
