@@ -12,38 +12,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Optional, Tuple
+from typing import Any, cast
 
 import numpy as np
 import jax.numpy as jnp
 
+import jax
 from jax import lax
 from jax.nn.initializers import zeros, lecun_normal
 from flax.linen.module import Module, compact
 from flax.linen.dtypes import promote_dtype
 
-from netket.utils import HashableArray, warn_deprecation, deprecate_dtype
+from netket.utils import HashableArray
 from netket.utils.types import Array, DType, NNInitFunc
 from netket.utils.group import PermutationGroup
-from typing import Sequence
+from collections.abc import Sequence
 from netket.graph import Graph, Lattice
+from netket.errors import SymmModuleInvalidInputShape
 
 # All layers defined here have kernels of shape [out_features, in_features, n_symm]
 default_equivariant_initializer = lecun_normal(in_axis=1, out_axis=0)
 
 
-def symm_input_warning(x_shape, new_x_shape, name):
-    warn_deprecation(
-        (
-            f"{len(x_shape)}-dimensional input to {name} layer is deprecated.\n"
-            f"Input shape {x_shape} has been reshaped to {new_x_shape}, where "
-            "the middle dimension encodes different input channels.\n"
-            "Please provide a 3-dimensional input.\nThis warning will become an "
-            "error in the future."
-        )
-    )
-
-
 class DenseSymmMatrix(Module):
     r"""Implements a symmetrized linear transformation over a permutation group
     using matrix multiplication."""
@@ -56,7 +46,7 @@ class DenseSymmMatrix(Module):
     """The number of output features. Will be the second dimension of the output."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
-    mask: Optional[HashableArray] = None
+    mask: HashableArray | None = None
     """Optional array of shape `(n_sites,)` used to restrict the convolutional
         kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
         boolean mask should be used"""
@@ -86,18 +76,11 @@ class DenseSymmMatrix(Module):
         Returns:
           The transformed input.
         """
-        # infer in_features and ensure input dimensions (batch, in_features,n_sites)
-
-        # TODO: Deprecated: Eventually remove and error if less than 3 dimensions
+        # ensure input dimensions (batch, in_features,n_sites)
         if x.ndim < 3:
-            old_shape = x.shape
-            if x.ndim == 1:
-                x = jnp.expand_dims(x, (0, 1))
-            elif x.ndim == 2:
-                x = jnp.expand_dims(x, 1)
-            symm_input_warning(old_shape, x.shape, "DenseSymm")
+            raise SymmModuleInvalidInputShape("DenseSymmMatrix", x)
 
-        in_features = x.shape[1]
+        in_features = x.shape[-2]
 
         if self.use_bias:
             bias = self.param(
@@ -141,7 +124,7 @@ class DenseSymmMatrix(Module):
             precision=self.precision,
         )
 
-        if self.use_bias:
+        if bias is not None:
             # Convert symmetry-reduced bias of shape (features,) to the full bias of
             # shape (..., features, 1).
             x += jnp.expand_dims(bias, 1)
@@ -156,11 +139,11 @@ class DenseSymmFFT(Module):
     """Array that lists the space group as permutations"""
     features: int
     """The number of output features. Will be the second dimension of the output."""
-    shape: Tuple
+    shape: tuple
     """Tuple that corresponds to shape of lattice"""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
-    mask: Optional[HashableArray] = None
+    mask: HashableArray | None = None
     """Optional array of shape `(n_sites,)` used to restrict the convolutional
         kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
         boolean mask should be used"""
@@ -176,8 +159,9 @@ class DenseSymmFFT(Module):
     def setup(self):
         sg = np.asarray(self.space_group)
 
-        self.n_cells = np.product(np.asarray(self.shape))
-        self.n_point = len(sg) // self.n_cells
+        self.n_cells = np.prod(np.asarray(self.shape))
+        self.n_symm = len(sg)
+        self.n_point = self.n_symm // self.n_cells
         self.sites_per_cell = sg.shape[1] // self.n_cells
 
         if self.mask is not None:
@@ -198,19 +182,14 @@ class DenseSymmFFT(Module):
         dimensions (-2: features, -1: group elements)
         """
 
-        # TODO: Deprecated: Eventually remove and error if less than 3 dimensions
-        # infer in_features and ensure input dimensions (batch, in_features,n_sites)
+        # ensure input dimensions (batch, in_features,n_sites)
         if x.ndim < 3:
-            old_shape = x.shape
-            if x.ndim == 1:
-                x = jnp.expand_dims(x, (0, 1))
-            elif x.ndim == 2:
-                x = jnp.expand_dims(x, 1)
-            symm_input_warning(old_shape, x.shape, "DenseSymm")
+            raise SymmModuleInvalidInputShape("DenseSymmMatrix", x)
 
-        in_features = x.shape[1]
+        in_features = x.shape[-2]
+        batch_shape = x.shape[:-2]
 
-        x = x.reshape(*x.shape[:-1], self.n_cells, self.sites_per_cell)
+        x = x.reshape(-1, in_features, self.n_cells, self.sites_per_cell)
         x = x.transpose(0, 1, 3, 2)
         x = x.reshape(*x.shape[:-1], *self.shape)
 
@@ -265,10 +244,11 @@ class DenseSymmFFT(Module):
         x = x.reshape(*x.shape[:3], *self.shape)
 
         x = jnp.fft.ifftn(x, s=self.shape).reshape(*x.shape[:3], self.n_cells)
-        x = x.transpose(0, 1, 3, 2).reshape(*x.shape[:2], -1)
+        x = x.transpose(0, 1, 3, 2)
+        x = x.reshape(*batch_shape, self.features, self.n_symm)
 
-        if self.use_bias:
-            x += jnp.expand_dims(bias, (0, 2))
+        if bias is not None:
+            x += jnp.expand_dims(bias, 1)
 
         if jnp.can_cast(x, dtype):
             return x
@@ -278,20 +258,23 @@ class DenseSymmFFT(Module):
 
 class DenseEquivariantFFT(Module):
     r"""Implements a group convolution using a fast fourier transform over the translation group.
+
     The group convolution can be written in terms of translational convolutions with
-    symmetry transformed filters as described in ` Cohen et. *al* <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
+    symmetry transformed filters as described in
+    `Cohen et. al <http://proceedings.mlr.press/v48/cohenc16.pdf>_`
+
     The translational convolutions are then implemented with Fast Fourier Transforms.
     """
 
     product_table: HashableArray
-    """ product table for space group"""
+    """Product table for space group."""
     features: int
     """The number of output features. Will be the second dimension of the output."""
-    shape: Tuple
+    shape: tuple
     """Tuple that corresponds to shape of lattice"""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
-    mask: Optional[HashableArray] = None
+    mask: HashableArray | None = None
     """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
         used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
         For best performance a boolean mask should be used"""
@@ -306,11 +289,11 @@ class DenseEquivariantFFT(Module):
     """Initializer for the bias. Defaults to zero initialization."""
 
     def setup(self):
-
         pt = np.asarray(self.product_table)
 
-        self.n_cells = np.product(np.asarray(self.shape))
-        self.n_point = len(pt) // self.n_cells
+        self.n_symm = len(pt)
+        self.n_cells = np.prod(np.asarray(self.shape))
+        self.n_point = self.n_symm // self.n_cells
         if self.mask is not None:
             (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
@@ -329,8 +312,9 @@ class DenseEquivariantFFT(Module):
         dimensions (-2: features, -1: group elements)
         """
         in_features = x.shape[-2]
+        batch_shape = x.shape[:-2]
 
-        x = x.reshape(*x.shape[:-1], self.n_cells, self.n_point)
+        x = x.reshape(-1, in_features, self.n_cells, self.n_point)
         x = x.transpose(0, 1, 3, 2)
         x = x.reshape(*x.shape[:-1], *self.shape)
 
@@ -384,10 +368,10 @@ class DenseEquivariantFFT(Module):
 
         x = jnp.fft.ifftn(x, s=self.shape).reshape(*x.shape[:3], self.n_cells)
         x = x.transpose(0, 1, 3, 2)
-        x = x.reshape(*x.shape[:2], -1)
+        x = x.reshape(*batch_shape, self.features, self.n_symm)
 
-        if self.use_bias:
-            x += jnp.expand_dims(bias, (0, 2))
+        if bias is not None:
+            x += jnp.expand_dims(bias, 1)
 
         if jnp.can_cast(x, dtype):
             return x
@@ -421,7 +405,7 @@ class DenseEquivariantIrrep(Module):
     stands for matrix multiplication.
     """
 
-    irreps: Tuple[HashableArray]
+    irreps: tuple[HashableArray, ...]
     """Irrep matrices of the symmetry group. Each element of the list is an
     array of shape [n_symm, d, d]; irreps[i][j] is the representation of the
     jth group element in irrep #i."""
@@ -429,7 +413,7 @@ class DenseEquivariantIrrep(Module):
     """The number of output features. Will be the second dimension of the output."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
-    mask: Optional[HashableArray] = None
+    mask: HashableArray | None = None
     """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
         used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
         For best performance a boolean mask should be used"""
@@ -486,7 +470,7 @@ class DenseEquivariantIrrep(Module):
             for i, shape in enumerate(shapes)
         )
 
-    def forward_ft(self, inputs: Array) -> Tuple[Array]:
+    def forward_ft(self, inputs: Array) -> tuple[Array, ...]:
         r"""Performs a forward group Fourier transform on the input.
         This is defined by
 
@@ -502,7 +486,7 @@ class DenseEquivariantIrrep(Module):
         """
         return self.disassemble(jnp.tensordot(inputs, self.forward, axes=1))
 
-    def inverse_ft(self, inputs: Tuple[Array]) -> Array:
+    def inverse_ft(self, inputs: tuple[Array, ...]) -> Array:
         r"""Performs an inverse group Fourier transform on the input.
         This is defined by
 
@@ -528,6 +512,8 @@ class DenseEquivariantIrrep(Module):
         dimensions (-2: features, -1: group elements)
         """
         in_features = x.shape[-2]
+        batch_shape = x.shape[:-2]
+        x = x.reshape(-1, in_features, self.n_symm)
 
         if self.use_bias:
             bias = self.param(
@@ -559,21 +545,22 @@ class DenseEquivariantIrrep(Module):
         x, kernel, bias = promote_dtype(x, kernel, bias, dtype=None)
         dtype = x.dtype
 
-        x = self.forward_ft(x)
+        x_fourier = self.forward_ft(x)
 
-        kernel = self.forward_ft(kernel)
+        kernel_fourier = self.forward_ft(kernel)
 
-        x = tuple(
+        x_fourier = tuple(
             lax.dot_general(
-                x[i], kernel[i], (((1, 4), (1, 3)), ((2,), (2,)))
+                x_fourier[i], kernel_fourier[i], (((1, 4), (1, 3)), ((2,), (2,)))
             ).transpose(1, 3, 0, 2, 4)
-            for i in range(len(x))
+            for i in range(len(x_fourier))
         )
 
-        x = self.inverse_ft(x)
+        x = self.inverse_ft(x_fourier).reshape(*batch_shape, self.features, self.n_symm)
+        x = cast(jax.Array, x)
 
-        if self.use_bias:
-            x += jnp.expand_dims(bias, (0, 2))
+        if bias is not None:
+            x += jnp.expand_dims(bias, 1)
 
         if jnp.can_cast(x, dtype):
             return x
@@ -586,15 +573,13 @@ class DenseEquivariantMatrix(Module):
     by multiplying by the full kernel matrix"""
 
     product_table: HashableArray
-    """Flattened product table generated by PermutationGroup.product_table().ravel()
-    that specifies the product of the group with its involution, or the
-    PermutationGroup object itself"""
+    """Product table for the space group."""
     features: int
     """The number of symmetry-reduced output features. The full output size
     is n_symm*features."""
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
-    mask: Optional[HashableArray] = None
+    mask: HashableArray | None = None
     """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
         used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
         For best performance a boolean mask should be used"""
@@ -616,13 +601,16 @@ class DenseEquivariantMatrix(Module):
     @compact
     def __call__(self, x: Array) -> Array:
         """Applies the equivariant transform to the inputs along the last dimension.
+
         Args:
           x: The nd-array to be transformed.
+
         Returns:
           The transformed input.
         """
         in_features = x.shape[-2]
 
+        kernel: jax.Array
         if self.mask is not None:
             kernel_params = self.param(
                 "kernel",
@@ -665,15 +653,12 @@ class DenseEquivariantMatrix(Module):
             precision=self.precision,
         )
 
-        x = x.reshape(-1, self.features, self.n_symm)
-
-        if self.use_bias:
+        if bias is not None:
             x += jnp.expand_dims(bias, 1)
 
         return x
 
 
-@deprecate_dtype
 def DenseSymm(
     symmetries, point_group=None, mode="auto", shape=None, mask=None, **kwargs
 ):
@@ -754,10 +739,9 @@ def DenseSymm(
         )
 
 
-@deprecate_dtype
 def DenseEquivariant(
     symmetries,
-    features: int = None,
+    features: int,
     mode="auto",
     shape=None,
     point_group=None,
@@ -767,11 +751,15 @@ def DenseEquivariant(
 ):
     r"""A group convolution operation that is equivariant over a symmetry group.
 
-    Acts on a feature map of symmetry poses of shape [num_samples, in_features, num_symm]
-    and returns a feature  map of poses of shape [num_samples, features, num_symm]
+    Acts on a feature map of symmetry poses of shape
+    :code:`[num_samples, in_features, num_symm]`
+    and returns a feature  map of poses of shape
+    :code:`[num_samples, features, num_symm]`
 
-    G-convolutions are described in ` Cohen et. {\it al} <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
-    and applied to quantum many-body problems in ` Roth et. {\it al} <https://arxiv.org/pdf/2104.05085.pdf>`_
+    G-convolutions are described in
+    `Cohen et. Al <http://proceedings.mlr.press/v48/cohenc16.pdf>`_
+    and applied to quantum many-body problems in
+    `Roth et. Al <https://arxiv.org/pdf/2104.05085.pdf>`_
 
     The G-convolution generalizes the convolution to non-commuting groups:
 
@@ -798,9 +786,10 @@ def DenseEquivariant(
         features: The number of output features. The full output shape
             is [n_batch,features,n_symm].
         use_bias: A bool specifying whether to add a bias to the output (default: True).
-        mask: Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
-        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
-        For best performance a boolean mask should be used.
+        mask: Optional array of shape :code:`(n_symm,)` where
+            :code:`(n_symm,) = len(graph.automorphisms())` used to restrict
+            the convolutional kernel. Only parameters with mask :math:'\ne 0' are
+            used. For best performance a boolean mask should be used.
         param_dtype: The datatype of the weights. Defaults to a 64bit float.
         precision: Optional argument specifying numerical precision of the computation.
             see :class:`jax.lax.Precision` for details.
@@ -810,28 +799,6 @@ def DenseEquivariant(
     if mask is not None:
         mask = HashableArray(mask)
 
-    # deprecate in_features
-    if in_features is not None:
-        warn_deprecation(
-            (
-                "`in_features` is now automatically detected from the input and deprecated."
-                "Please remove it when calling `DenseEquivariant`."
-            )
-        )
-    if "out_features" in kwargs:
-        warn_deprecation(
-            "`out_features` has been renamed to `features` and the old name is "
-            "now deprecated. Please update your code."
-        )
-        if features is not None:
-            raise ValueError(
-                "You must only specify `features`. `out_features` is deprecated."
-            )
-        features = kwargs.pop("out_features")
-
-    if features is None:
-        raise ValueError("`features` not specified (the number of output features).")
-
     kwargs["features"] = features
 
     if isinstance(symmetries, Lattice) and (
@@ -858,9 +825,10 @@ def DenseEquivariant(
         sg = symmetries
 
     elif isinstance(symmetries, Sequence):
+        symmetries = cast(Sequence[HashableArray], symmetries)
         if mode not in ["irreps", "auto"]:
             raise ValueError("Specification of symmetries incompatible with mode")
-        return DenseEquivariantIrrep(symmetries, mask=mask, **kwargs)
+        return DenseEquivariantIrrep(tuple(symmetries), mask=mask, **kwargs)
     else:
         if symmetries.ndim == 2 and symmetries.shape[0] == symmetries.shape[1]:
             if mode == "irreps":
@@ -889,14 +857,14 @@ def DenseEquivariant(
             )
         else:
             return DenseEquivariantFFT(
-                HashableArray(sg.product_table), mask=mask, shape=shape, **kwargs
+                HashableArray(sg.product_table), mask=mask, shape=shape, **kwargs  # type: ignore
             )
     elif mode in ["irreps", "auto"]:
         irreps = tuple(HashableArray(irrep) for irrep in sg.irrep_matrices())
         return DenseEquivariantIrrep(irreps, mask=mask, **kwargs)
     elif mode == "matrix":
         return DenseEquivariantMatrix(
-            HashableArray(sg.product_table), mask=mask, **kwargs
+            HashableArray(sg.product_table), mask=mask, **kwargs  # type: ignore
         )
     else:
         raise ValueError(
