@@ -18,7 +18,7 @@ from ._discrete_operator import DiscreteOperator
 class SpecialHamiltonian(DiscreteOperator):
     def to_local_operator(self):
         raise NotImplementedError(
-            "Must implemented to_local_operator for {}".format(type(self))
+            f"Must implemented to_local_operator for {type(self)}"
         )
 
     def conjugate(self, *, concrete: bool = True):
@@ -27,46 +27,41 @@ class SpecialHamiltonian(DiscreteOperator):
     def __add__(self, other):
         if type(self) is type(other):
             res = self.copy()
-            res += other
-            return res
-
-        return self.to_local_operator().__add__(other)
+            res = res.__iadd__(other)
+            if res is not NotImplemented:
+                return res
+            else:
+                return self.to_local_operator() + other.to_local_operator()
+        return self.to_local_operator() + other
 
     def __sub__(self, other):
         if type(self) is type(other):
             res = self.copy()
-            res -= other
-            return res
+            res = res.__isub__(other)
+            if res is not NotImplemented:
+                return res
+            else:
+                return self.to_local_operator() - other.to_local_operator()
 
-        return self.to_local_operator().__sub__(other)
+        return self.to_local_operator() - other
 
     def __radd__(self, other):
-        if type(self) is type(other):
-            res = self.copy()
-            res += other
-            return res
-
-        return self.to_local_operator().__radd__(other)
+        return other + self.to_local_operator()
 
     def __rsub__(self, other):
-        if type(self) is type(other):
-            res = self.copy()
-            res -= other
-            return res
-
-        return self.to_local_operator().__rsub__(other)
+        return other - self.to_local_operator()
 
     def __iadd__(self, other):
         if type(self) is type(other):
-            self._iadd_same_hamiltonian(other)
-            return self
+            res = self._iadd_same_hamiltonian(other)
+            return res
 
         return NotImplemented
 
     def __isub__(self, other):
         if type(self) is type(other):
-            self._isub_same_hamiltonian(other)
-            return self
+            res = self._isub_same_hamiltonian(other)
+            return res
 
         return NotImplemented
 
@@ -81,6 +76,9 @@ class SpecialHamiltonian(DiscreteOperator):
             other = other.to_local_operator()
         return self.to_local_operator().__matmul__(other)
 
+    def __neg__(self):
+        return -1 * self.to_local_operator()
+
     def _op__rmatmul__(self, other):
         if hasattr(other, "to_local_operator"):
             other = other.to_local_operator()
