@@ -11,28 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import Optional, Callable, Union, List
-from functools import partial
+from typing import Hashable
+from collections.abc import Callable
 
 import numpy as np
 
 import jax
 import jax.numpy as jnp
 
+from netket.utils import struct
 from netket.utils.types import DType, PyTree, Array
 import netket.jax as nkjax
 from netket.hilbert import AbstractHilbert
 from netket.operator import ContinuousOperator
+from netket.utils import HashableArray
 
 
 def jacrev(f):
     def jacfun(x):
         y, vjp_fun = nkjax.vjp(f, x)
         if y.size == 1:
-            eye = jnp.eye(y.size)[0]
+            eye = jnp.eye(y.size, dtype=x.dtype)[0]
             J = jax.vmap(vjp_fun, in_axes=0)(eye)
         else:
-            eye = jnp.eye(y.size)
+            eye = jnp.eye(y.size, dtype=x.dtype)
             J = jax.vmap(vjp_fun, in_axes=0)(eye)
         return J
 
@@ -42,7 +44,7 @@ def jacrev(f):
 def jacfwd(f):
     def jacfun(x):
         jvp_fun = lambda s: jax.jvp(f, (x,), (s,))[1]
-        eye = jnp.eye(len(x))
+        eye = jnp.eye(len(x), dtype=x.dtype)
         J = jax.vmap(jvp_fun, in_axes=0)(eye)
         return J
 
@@ -54,20 +56,22 @@ class KineticEnergy(ContinuousOperator):
     :math:`E_{kin} = -1/2 ( \sum_i \frac{1}{m_i} (\log(\psi))'^2 + (\log(\psi))'' )`
     """
 
+    _is_hermitian: bool = struct.static_field()
+    _mass: Array = struct.field()
+
     def __init__(
         self,
         hilbert: AbstractHilbert,
-        mass: Union[float, List[float]],
-        dtype: Optional[DType] = None,
+        mass: float | list[float],
+        dtype: DType | None = None,
     ):
         r"""Args:
         hilbert: The underlying Hilbert space on which the operator is defined
         mass: float if all masses are the same, list indicating the mass of each particle otherwise
-        dtype: Data type of the matrix elements. Defaults to `np.float64`
+        dtype: Data type of the mass
         """
 
         self._mass = jnp.asarray(mass, dtype=dtype)
-
         self._is_hermitian = np.allclose(self._mass.imag, 0.0)
 
         super().__init__(hilbert, self._mass.dtype)
@@ -81,26 +85,35 @@ class KineticEnergy(ContinuousOperator):
         return self._is_hermitian
 
     def _expect_kernel_single(
-        self, logpsi: Callable, params: PyTree, x: Array, mass: Optional[PyTree]
+        self,
+        logpsi: Callable,
+        params: PyTree,
+        x: Array,
     ):
         def logpsi_x(x):
             return logpsi(params, x)
 
         dlogpsi_x = jacrev(logpsi_x)
+        inverse_mass = jnp.reciprocal(self._mass)
 
         dp_dx2 = jnp.diag(jacfwd(dlogpsi_x)(x)[0].reshape(x.shape[0], x.shape[0]))
         dp_dx = dlogpsi_x(x)[0][0] ** 2
 
-        return -0.5 * jnp.sum(mass * (dp_dx2 + dp_dx), axis=-1)
+        return -0.5 * jnp.sum(inverse_mass * (dp_dx2 + dp_dx), axis=-1)
 
-    @partial(jax.vmap, in_axes=(None, None, None, 0, None))
     def _expect_kernel(
-        self, logpsi: Callable, params: PyTree, x: Array, coefficient: Optional[PyTree]
+        self,
+        logpsi: Callable,
+        params: PyTree,
+        x: Array,
     ):
-        return self._expect_kernel_single(logpsi, params, x, coefficient)
+        return jax.vmap(
+            lambda x_: self._expect_kernel_single(logpsi, params, x_), in_axes=(0)
+        )(x)
 
-    def _pack_arguments(self) -> PyTree:
-        return 1.0 / self._mass
+    @struct.property_cached(pytree_ignore=True)
+    def _attrs(self) -> tuple[Hashable, ...]:
+        return (self.hilbert, self.dtype, HashableArray(self.mass))
 
     def __repr__(self):
         return f"KineticEnergy(m={self._mass})"
