@@ -12,20 +12,34 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any
+from typing import Any, TYPE_CHECKING
 from functools import partial
 
 import jax
+from plum import Callable
 
 import netket.jax as nkjax
+from netket.optimizer.linear_operator import LinearOperator
 
-from .qgt_jacobian_dense import QGTJacobianDense
-from .qgt_jacobian_pytree import QGTJacobianPyTree
+from .qgt_jacobian import QGTJacobianDense, QGTJacobianPyTree
 from .qgt_onthefly import QGTOnTheFly
 
-from ..solver import cholesky, svd, LU, solve
+from .. import solver as nk_solver_module
 
-solvers = [cholesky, svd, LU, solve]
+if TYPE_CHECKING:
+    from netket.vqs import VariationalState
+
+QGTConstructor = Callable[["VariationalState"], LinearOperator]
+
+solvers: list[Callable] = []
+
+for solver in dir(nk_solver_module):
+    # only add solvers, not random
+    # useless things
+    if solver[:2] == "__":
+        continue
+    else:
+        solvers.append(getattr(nk_solver_module, solver))
 
 
 def _is_dense_solver(solver: Any) -> bool:
@@ -41,13 +55,15 @@ def _is_dense_solver(solver: Any) -> bool:
     return False
 
 
-def default_qgt_matrix(variational_state, solver=False, **kwargs):
+def default_qgt_matrix(
+    variational_state, solver: Any = False, **kwargs
+) -> QGTConstructor:
     """
     Determines default metric tensor depending on variational_state and solver
     """
-    from netket.vqs import ExactState
+    from netket.vqs import FullSumState
 
-    if isinstance(variational_state, ExactState):
+    if isinstance(variational_state, FullSumState):
         return partial(QGTJacobianPyTree, **kwargs)
 
     n_param_leaves = len(jax.tree_util.tree_leaves(variational_state.parameters))
@@ -58,18 +74,18 @@ def default_qgt_matrix(variational_state, solver=False, **kwargs):
         return partial(QGTJacobianDense, **kwargs)
 
     # TODO: Remove this once all QGT support diag_scale.
-    has_diag_rescale = kwargs.pop("diag_scale", None) is not None
+    has_diag_rescale = kwargs.get("diag_scale") is not None
 
     # arbitrary heuristic: if the network's parameters has many leaves
     # (an rbm has 3) then JacobianDense might be faster
     # the numbers chosen below are rather arbitrary and should be tuned.
     if (n_param_leaves > 6 and n_params > 800) or has_diag_rescale:
-        if nkjax.tree_ishomogeneous(variational_state.variables):
+        if nkjax.tree_ishomogeneous(variational_state.parameters):
             return partial(QGTJacobianDense, **kwargs)
         else:
             return partial(QGTJacobianPyTree, **kwargs)
     else:
-        return partial(QGTOnTheFly, **kwargs)
+        return partial(QGTOnTheFly, **kwargs)  # type: ignore
 
 
 class QGTAuto:
@@ -87,22 +103,26 @@ class QGTAuto:
     QGT to chose.
     """
 
-    _last_matrix = None
+    _last_matrix: QGTConstructor | None = None
     """
     Cached last QGT. Used when vstate == _last_vstate
     """
 
-    _kwargs = {}
+    _kwargs: dict = {}
     """
     Kwargs passed at construction. Used when constructing a QGT.
     """
 
-    def __init__(self, solver=None, **kwargs):
+    _solver: Callable | None
+
+    def __init__(self, solver: Callable | None = None, **kwargs):
         self._solver = solver
 
         self._kwargs = kwargs
 
-    def __call__(self, variational_state, *args, **kwargs):
+    def __call__(
+        self, variational_state: "VariationalState", *args, **kwargs
+    ) -> LinearOperator:
         if self._last_vstate != variational_state:
             self._last_vstate = variational_state
 
@@ -110,7 +130,7 @@ class QGTAuto:
                 variational_state, solver=self._solver, **self._kwargs, **kwargs
             )
 
-        return self._last_matrix(variational_state, *args, **kwargs)
+        return self._last_matrix(variational_state, *args, **kwargs)  # type: ignore
 
     def __repr__(self):
         return "QGTAuto()"
