@@ -15,10 +15,6 @@
 from .base import (
     Sampler,
     SamplerState,
-    sampler_state,
-    reset,
-    sample,
-    samples,
 )
 
 from .exact import ExactSampler
@@ -32,7 +28,14 @@ from .metropolis import (
     MetropolisHamiltonian,
     MetropolisGaussian,
     MetropolisAdjustedLangevin,
-    sample_next,
+    MetropolisFermionHop,
+)
+
+from .parallel_tempering import (
+    ParallelTemperingSampler,
+    ParallelTemperingLocal,
+    ParallelTemperingExchange,
+    ParallelTemperingHamiltonian,
 )
 
 from .metropolis_numpy import (
