@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Optional, Tuple
+from typing import Any
 import abc
 
 from flax import linen as nn
@@ -28,8 +28,7 @@ if config.netket_sphinx_build:
     from netket import sampler
 
 
-@struct.dataclass
-class MetropolisRule(abc.ABC):
+class MetropolisRule(struct.Pytree):
     """
     Base class for transition rules of Metropolis, such as Local, Exchange, Hamiltonian
     and several others.
@@ -41,7 +40,7 @@ class MetropolisRule(abc.ABC):
         machine: nn.Module,
         params: PyTree,
         key: PRNGKeyT,
-    ) -> Optional[Any]:
+    ) -> Any | None:
         """
         Initialises the optional internal state of the Metropolis sampler transition
         rule.
@@ -67,7 +66,7 @@ class MetropolisRule(abc.ABC):
         machine: nn.Module,
         params: PyTree,
         sampler_state: "sampler.SamplerState",  # noqa: F821
-    ) -> Optional[Any]:
+    ) -> Any | None:
         """
         Resets the internal state of the Metropolis Sampler Transition Rule.
 
@@ -94,7 +93,7 @@ class MetropolisRule(abc.ABC):
         sampler_state: "sampler.SamplerState",  # noqa: F821
         key: PRNGKeyT,
         σ: jnp.ndarray,
-    ) -> Tuple[jnp.ndarray, Optional[jnp.ndarray]]:
+    ) -> tuple[jnp.ndarray, jnp.ndarray | None]:
         r"""
         Proposes a new configuration set of configurations $\sigma'$ starting from the current
         chain configurations :math:`\sigma`.
@@ -119,7 +118,6 @@ class MetropolisRule(abc.ABC):
            A tuple containing the new configurations :math:`\sigma'` and the optional vector of
            log corrections to the transition probability.
         """
-        pass
 
     def random_state(
         self,
