@@ -13,13 +13,13 @@
 # limitations under the License.
 
 import math
-from typing import Union
 
 import jax
 import numpy as np
 from jax import numpy as jnp
 
-from netket.utils import config, mpi, struct
+from netket.utils import config, struct
+from netket.jax.sharding import extract_replicated
 
 from . import mean as _mean
 from . import var as _var
@@ -27,20 +27,52 @@ from . import total_size as _total_size
 from ._autocorr import integrated_time
 
 
-def _format_decimal(value, std, var):
-    if math.isfinite(std) and std > 1e-7:
-        decimals = max(int(np.ceil(-np.log10(std))), 0)
-        return (
-            "{0:.{1}f}".format(value, decimals + 1),
-            "{0:.{1}f}".format(std, decimals + 1),
-            "{0:.{1}f}".format(var, decimals + 1),
-        )
+def _format_main_string(value, std, var):
+
+    if not math.isfinite(abs(value)) or not math.isfinite(std):
+        return f"{value:.2e} ± {std:.2e} [σ²={var:.1e}"
+
+    elif std == 0.0:
+        return f"{value:.3e} [σ²={var:.1e}"
+
+    elif (abs(value) + abs(std) < 1e-2) or abs(std) <= 1e-7:
+        value_std_str = _format_scientific_notation(value, std)
+        return value_std_str + f" [σ²={var:.1e}"
+
     else:
-        return (
-            "{0:.3e}".format(value),
-            "{0:.3e}".format(std),
-            "{0:.3e}".format(var),
-        )
+        if std < 1e-15:
+            decimals = 15
+        else:
+            decimals = max(int(np.ceil(-np.log10(std))), 0) + 1
+        return f"{value:.{decimals}f} ± {std:.{decimals}f} [σ²={var:.1e}"
+
+
+def _format_scientific_notation(value, std):
+
+    length = 5
+
+    value_ref = (
+        np.real(value)
+        if np.abs(np.real(value)) > np.abs(np.imag(value))
+        else np.imag(value)
+    )
+
+    exponent_val = int(np.floor(np.log10(np.abs(value_ref))))
+    exponent_std = int(np.floor(np.log10(std)))
+    n_digits = max(exponent_val - exponent_std, 0) + 1
+
+    mantissa_val = value / 10**exponent_val
+    mantissa_std = std / 10**exponent_val
+
+    value_length = 2 * length + 2 if isinstance(value, complex) else length
+
+    mantissa_str = (
+        f"{mantissa_val:.{n_digits}f}".rjust(value_length)
+        + " ± "
+        + f"{mantissa_std:.{n_digits}f}".rjust(length)
+    )
+
+    return mantissa_str + " e" + f"{value_ref:e}".split("e")[1]
 
 
 _NaN = float("NaN")
@@ -57,7 +89,7 @@ def _maybe_item(x):
 class Stats:
     """A dict-compatible pytree containing the result of the statistics function."""
 
-    mean: Union[float, complex] = _NaN
+    mean: float | complex = _NaN
     """The mean value."""
     error_of_mean: float = _NaN
     """Estimate of the error of the mean."""
@@ -108,15 +140,21 @@ class Stats:
         return "Mean", self.to_dict()
 
     def __repr__(self):
-        mean, err, var = _format_decimal(self.mean, self.error_of_mean, self.variance)
+        # extract adressable data from fully replicated arrays
+        self = extract_replicated(self)
+        main_string = _format_main_string(
+            _maybe_item(self.mean),
+            _maybe_item(self.error_of_mean),
+            _maybe_item(self.variance),
+        )
         if not math.isnan(self.R_hat):
-            ext = ", R̂={:.4f}".format(self.R_hat)
+            ext = f", R̂={self.R_hat:.3f}"
         else:
             ext = ""
         if config.netket_experimental_fft_autocorrelation:
             if not (math.isnan(self.tau_corr) and math.isnan(self.tau_corr_max)):
-                ext += ", τ={:.1f}<{:.1f}".format(self.tau_corr, self.tau_corr_max)
-        return "{} ± {} [σ²={}{}]".format(mean, err, var, ext)
+                ext += f", τ={self.tau_corr:.1f}<{self.tau_corr_max:.1f}"
+        return main_string + f"{ext}]"
 
     # Alias accessors
     def __getattr__(self, name):
@@ -133,9 +171,7 @@ class Stats:
         elif name in ("tau_corr_max", "TauCorrMax"):
             return self.tau_corr_max
         else:
-            raise AttributeError(
-                "'Stats' object object has no attribute '{}'".format(name)
-            )
+            raise AttributeError(f"'Stats' object object has no attribute '{name}'")
 
     def real(self):
         return self.replace(mean=np.real(self.mean))
@@ -193,12 +229,11 @@ def _split_R_hat(data, W):
 BLOCK_SIZE = 32
 
 
-def statistics(data):
+def statistics(data: jax.Array) -> Stats:
     r"""
     Returns statistics of a given array (or matrix, see below) containing a stream of data.
     This is particularly useful to analyze Markov Chain data, but it can be used
     also for other type of time series.
-    Assumes same shape on all MPI processes.
 
     Args:
         data (vector or matrix): The input data. It can be real or complex valued.
@@ -251,8 +286,8 @@ def _statistics(data):
     variance = _var(data)
 
     taus = jax.vmap(integrated_time)(data)
-    tau_avg, _ = mpi.mpi_mean_jax(jnp.mean(taus))
-    tau_max, _ = mpi.mpi_max_jax(jnp.max(taus))
+    tau_avg = jnp.mean(taus)
+    tau_max = jnp.max(taus)
 
     batch_var, n_batches = _batch_variance(data)
     if n_batches > 1:
