@@ -12,11 +12,5 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Union
 
-from plum import dispatch, parametric, convert, Val  # noqa: F401
-
-# Signature-types for True and False
-TrueT = Val[True]
-FalseT = Val[False]
-Bool = Union[TrueT, FalseT]
+from plum import dispatch, overload, parametric, convert  # noqa: F401
