@@ -16,7 +16,7 @@
 # pylint: disable=function-redefined
 
 import itertools
-from typing import List, Tuple
+from typing import overload, Literal, Any
 
 import numpy as np
 
@@ -25,7 +25,7 @@ from netket.utils.float import comparable, prune_zeros
 from netket.utils.types import Array
 from netket.utils.dispatch import dispatch
 
-from ._semigroup import Element, FiniteSemiGroup, Identity
+from netket.utils.group._semigroup import Element, FiniteSemiGroup, Identity
 
 
 @struct.dataclass
@@ -74,7 +74,17 @@ class FiniteGroup(FiniteSemiGroup):
             for index, element in enumerate(self.elems)
         }
 
-    def remove_duplicates(self, *, return_inverse=False) -> "FiniteGroup":
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[False] = False
+    ) -> "FiniteGroup": ...
+
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[True]
+    ) -> tuple["FiniteGroup", Any]: ...
+
+    def remove_duplicates(self, *, return_inverse=False):
         r"""
         Returns a new :class:`FiniteGroup` with duplicate elements (that is,
         elements with identical canonical forms) removed.
@@ -155,7 +165,7 @@ class FiniteGroup(FiniteSemiGroup):
     @struct.property_cached
     def conjugacy_table(self) -> Array:
         r"""
-        The conjugacy table of this Permutation Group.
+        The conjugacy table of the group.
 
         Assuming the definitions
 
@@ -172,7 +182,7 @@ class FiniteGroup(FiniteSemiGroup):
         return self.product_table[self.product_table, col_index]
 
     @struct.property_cached
-    def conjugacy_classes(self) -> Tuple[Array, Array, Array]:
+    def conjugacy_classes(self) -> tuple[Array, Array, Array]:
         r"""
         The conjugacy classes of the group.
 
@@ -205,46 +215,78 @@ class FiniteGroup(FiniteSemiGroup):
 
         return classes, representatives, inverse
 
-    @struct.property_cached
-    def character_table_by_class(self) -> Array:
+    def check_multiplier(self, multiplier: Array, rtol=1e-8, atol=0) -> bool:
         r"""
-        Calculates the character table using Burnside's algorithm.
+        Checks the associativity constraint of Schur multipliers.
 
-        Each row of the output lists the characters of one irrep in the order the
-        conjugacy classes are listed in `self.conjugacy_classes`.
+        .. math::
 
-        Assumes that :code:`Identity() == self[0]`, if not, the sign of some
-        characters may be flipped. The irreps are sorted by dimension.
+            \alpha(x, y) \alpha(xy, z) = \alpha(x, yz) \alpha(y, z).
+
+
+        Arguments:
+            multiplier: the array of Schur multipliers :math:`\alpha(x,y)`
+            rtol: relative tolerance
+            atol: absolute tolerance
+
+        Returns:
+            whether :code:`multiplier` is a valid Schur multiplier
+            up to the given tolerance
+
+        Raises:
+            :class:`ValueError`
+                if the shape of `multiplier` does not match
+                the size of the group
         """
+        if multiplier.shape != (len(self), len(self)):
+            raise ValueError(
+                "Schur multipliers must form a square array of the same size\n"
+                f"as the group ({len(self)}, {len(self)}), "
+                f"got {multiplier.shape} instead!"
+            )
+
+        PT = self.product_table[self.inverse]
+        # \alpha(x, y) \alpha(xy, z)
+        left = np.expand_dims(multiplier, 2) * multiplier[PT, :]
+        # \alpha(x, yz) \alpha(y, z)
+        right = multiplier[:, PT] * np.expand_dims(multiplier, 0)
+        return np.allclose(left, right, rtol=rtol, atol=atol)
+
+    def _character_from_class_matrix(
+        self, class_matrix: Array, which_class: Array | None = None
+    ) -> np.ndarray:
+        r"""Given a linear combination of class matrices,
+        diagonalise it and normalise the eigenvectors as characters:
+
+        .. math::
+
+            \sum_g |\chi(g)|^2 = \sum_C |C| |\chi(c_0)|^2 = |G|.
+
+        Arguments:
+            class_matrix: square matrix diagonalised by the characters
+                :math:`\chi(c_0)` (without any further scaling).
+            which_class: (optional) boolean array, specifying which
+                conjugacy classes are represented in `class_matrix`.
+
+        Returns:
+            irrep characters obtained from the eigenvectors of `class_matrix`.
+
+            Excluded classes are filled in with zeros. The irreps are
+            sorted by dimension.
+        """
+        _, table = np.linalg.eig(class_matrix)
+        table = table.T  # want eigenvectors as rows
+
+        # normalisation
         classes, _, _ = self.conjugacy_classes
         class_sizes = classes.sum(axis=1)
-        # Construct a random linear combination of the class matrices c_S
-        #    (c_S)_{RT} = #{r,s: r \in R, s \in S: rs = t}
-        # for conjugacy classes R,S,T, and a fixed t \in T.
-        #
-        # From our oblique times table it is easier to calculate
-        #    (d_S)_{RT} = #{r,t: r \in R, t \in T: rs = t}
-        # for a fixed s \in S. This is just `product_table == s`, aggregated
-        # over conjugacy classes. c_S and d_S are related by
-        #    c_{RST} = |S| d_{RST} / |T|;
-        # since we only want a random linear combination, we forget about the
-        # constant |S| and only divide each column through with the appropriate |T|
-        class_matrix = (
-            classes @ random(len(self), seed=0)[self.product_table] @ classes.T
-        )
-        class_matrix /= class_sizes
-
-        # The vectors |R|\chi(r) are (column) eigenvectors of all class matrices
-        # the random linear combination ensures (with probability 1) that
-        # none of them are degenerate
-        _, table = np.linalg.eig(class_matrix)
-        table = table.T / class_sizes
-
-        # Normalise the eigenvectors by orthogonality: \sum_g |\chi(g)|^2 = |G|
-        norm = np.sum(np.abs(table) ** 2 * class_sizes, axis=1, keepdims=True) ** 0.5
+        if which_class is not None:
+            class_sizes = class_sizes[which_class]
+        norm = np.sum(np.abs(table) ** 2 * class_sizes, axis=1, keepdims=True)
+        norm = (norm / len(self)) ** 0.5
         table /= norm
-        table /= _cplx_sign(table[:, 0])[:, np.newaxis]  # ensure correct sign
-        table *= len(self) ** 0.5
+        # ensure correct sign (i.e. identity should have a real character)
+        table /= _cplx_sign(table[:, 0])[:, np.newaxis]
 
         # Sort lexicographically, ascending by first column, descending by others
         sorting_table = np.column_stack((table.real, table.imag))
@@ -256,46 +298,234 @@ class FiniteGroup(FiniteSemiGroup):
         # Get rid of annoying nearly-zero entries
         table = prune_zeros(table)
 
-        return table
+        if which_class is not None:
+            # fill in excluded classes
+            full_table = np.zeros((len(table), len(classes)), dtype=table.dtype)
+            full_table[:, which_class] = table
+            return full_table
+        else:
+            return table
 
-    def character_table(self) -> Array:
+    def projective_characters_by_class(
+        self, multiplier: Array | None
+    ) -> tuple[np.ndarray, np.ndarray]:
+        r"""
+        Calculates the character table of projective representations with a
+        given Schur multiplier α using a modified Burnside algorithm.
+
+        Arguments:
+            multiplier: the unitary Schur multiplier.
+                If unspecified, computes linear representation characters.
+
+        Returns:
+            - :code:`characters_by_class`
+                a 2D array, each row containing the characters of a
+                representative element of each conjugacy class in one
+                projective irrep with the given multiplier.
+            - :code:`class_factors`
+                a 1D array listing the "class factors" of each element of
+                the group. The character of each element is the product
+                of the character of the class representative with this
+                class factor.
+                (Only returned if :code:`multiplier` is not :code:`None`.)
+
+        Note: the algorithm and the definitions above are explained in more
+        detail in https://arxiv.org/abs/2505.14790.
+        """
+        # deal with trivial multipliers
+        if multiplier is None:
+            return self.character_table_by_class
+        elif np.allclose(multiplier, 1.0, rtol=0, atol=1e-8):
+            # still trivial but should conform to projective irrep return format
+            return self.character_table_by_class, np.ones(len(self))
+
+        # check unitarity
+        if not np.allclose(np.abs(multiplier), 1.0, rtol=0, atol=1e-8):
+            raise ValueError("Schur multiplier must be unitary")
+
+        # compute class factors
+        class_factors = np.zeros(len(self), dtype=multiplier.dtype)
+        classes, class_reps, _ = self.conjugacy_classes
+        reg_classes = np.zeros(len(class_reps), dtype=bool)
+
+        numerator = multiplier[self.inverse, np.arange(len(self))]
+        for i, (g, cls) in enumerate(zip(class_reps, classes)):
+            # β(g, h^-1 g h) = α(h^-1, h) / α(h^-1, g) α(h^-1 g, h)
+            β = (
+                numerator
+                / multiplier[self.inverse, g]
+                / multiplier[self.product_table[:, g], np.arange(len(self))]
+            )
+
+            class_size = cls.sum()
+            centralizer_size = len(self) // class_size
+            # collate sets of h with equal h^-1 g h
+            h_sets = np.argsort(self.conjugacy_table[g]).reshape(
+                class_size, centralizer_size
+            )
+            # average β across each set
+            β = np.average(β[h_sets], axis=1)
+
+            if np.allclose(np.abs(β), 1.0, rtol=0, atol=1e-8):
+                # if the class is α-regular, β for each averaged entry was equal
+                # and the array now contains unit complex numbers
+                class_factors[cls] = β
+                reg_classes[i] = True
+            elif not np.allclose(np.abs(β), 0.0, rtol=0, atol=1e-8):
+                # otherwise, the different β should average to zero
+                raise RuntimeError(
+                    "Class factors close to neither unity of zero\n" + repr(β)
+                )
+
+        # The algorithm hinges on the equation
+        #   \sum_C M_{ABC} \chi(c_0) = |A||B| \chi(a_0) \chi(b_0) / d_\chi,
+        # where
+        #   M_{ABC} = \sum_{a \in A, b \in B, ab \in C} α(a,b) β(ab)/β(a)β(b)
+        #
+        # From the oblique times table, we can easily extract the array
+        #   (M'_B)_{AC} = \sum_{a \in A, ab_0 \in C} α(a, b_0) β(ab_0)/β(a)
+        #               = M_{ABC} / |B|
+        # by building an array of α(a, b_0) β(ab_0)/β(a), filtering for
+        # `product_table == b_0`, and aggregating over conjugacy classes.
+        #
+        # Defining
+        #   (M_B)_{AC} := (M'_B)_{AC} / |A|,
+        # we finally have the eigenvalue equation
+        #   M_B \vec\chi = \chi(b_0)/d_\chi \vec\chi,
+        # where \vec\chi is the vector of characters \chi(c_0).
+
+        # Construct a random linear combination of the M_B
+        # array of α(a,b) β(ab)/β(a)
+        class_matrix = (
+            multiplier[np.arange(len(self))[:, None], self.product_table]
+            * class_factors
+            # set non-regular elements to zero instead of inf/nan
+            / np.where(class_factors == 0.0, np.inf, class_factors)[:, None]
+        )
+
+        # multiply with random weights for α-regular class representatives
+        # TODO should we have a term for every α-regular element?
+        weight_ = random(reg_classes.sum(), seed=0, cplx=np.iscomplexobj(multiplier))
+        weight = np.zeros(len(self), dtype=weight_.dtype)
+        weight[class_reps[reg_classes]] = weight_
+        class_matrix *= weight[self.product_table]
+
+        # aggregate by α-regular conjugacy class, divide by |A|
+        classes = classes[reg_classes]
+        class_matrix = classes @ class_matrix @ classes.T
+        class_matrix /= classes.sum(axis=1)[:, None]
+
+        return (
+            self._character_from_class_matrix(class_matrix, reg_classes),
+            class_factors,
+        )
+
+    @struct.property_cached
+    def character_table_by_class(self) -> np.ndarray:
         r"""
         Calculates the character table using Burnside's algorithm.
 
-        Each row of the output lists the characters of all group elements
-        for one irrep, i.e. :code:`self.character_table()[i,g]` gives
-        :math:`\chi_i(g)`.
+        Each row of the output lists the characters of one irrep in the order the
+        conjugacy classes are listed in :attr:`conjugacy_classes`.
 
-        Assumes that :code:`Identity() == self[0]`, if not, the sign
-        of some characters may be flipped. The irreps are sorted by dimension.
+        Assumes that :code:`Identity() == self[0]`, if not, the sign of some
+        characters may be flipped. The irreps are sorted by dimension.
         """
-        _, _, inverse = self.conjugacy_classes
-        CT = self.character_table_by_class
-        return CT[:, inverse]
+        classes, _, _ = self.conjugacy_classes
+        class_sizes = classes.sum(axis=1)
+        # Burnside's algorithm hinges on the equation
+        #   \sum_C M_{ABC} \chi_C = |A||B| \chi_A \chi_B / d_\chi,
+        # where
+        #   M_{ABC} = #{a \in A, b \in B: ab \in C}.
+        #
+        # From the oblique times table, we can easily extract the array
+        #   (M'_B)_{AC} = #{a \in A: ab_0 \in C}
+        # by aggregating `product_table == b_0` over conjugacy classes.
+        #
+        # Defining
+        #   (M_B)_{AC} := (M'_B)_{AC} / |A|,
+        # we finally have the eigenvalue equation
+        #   M_B \vec\chi = \chi_B / d_\chi \vec\chi,
+        # where \vec\chi is the vector of characters \chi_C.
 
-    def character_table_readable(self) -> Tuple[List[str], Array]:
+        # Construct a random linear combination of the M_B
+        # i.e. weight each `product_table==b` with some random weight w(b)
+        class_matrix = (
+            classes @ random(len(self), seed=0)[self.product_table] @ classes.T
+        )
+        class_matrix /= class_sizes[:, None]
+
+        return self._character_from_class_matrix(class_matrix)
+
+    def character_table(self, multiplier: Array | None = None) -> np.ndarray:
+        r"""
+        Calculates the character table using Burnside's algorithm.
+
+        Arguments:
+            multiplier: (optional) Schur multiplier
+
+        Returns:
+            a matrix of all linear irrep characters (if :code:`multiplier is None`)
+            or projective irrep characters with the given :code:`multiplier`,
+            sorted by dimension.
+
+            Each row of lists the characters of all group elements
+            for one irrep, i.e. :code:`self.character_table()[i,g]`
+            gives :math:`\chi_i(g)`.
+
+        It is assumed that :code:`Identity() == self[0]`. If not, the sign
+        of some characters may be flipped and the sorting by dimension
+        will be wrong.
+        """
+        _, _, class_idx = self.conjugacy_classes
+        if multiplier is None or np.allclose(multiplier, 1.0, rtol=0, atol=1e-8):
+            # linear representations
+            CT = self.character_table_by_class
+            return CT[:, class_idx]
+        else:
+            # projective representations
+            CT, class_factor = self.projective_characters_by_class(multiplier)
+            return CT[:, class_idx] * class_factor
+
+    def character_table_readable(
+        self, multiplier: Array | None = None, full: bool = False
+    ) -> tuple[list[str], Array]:
         r"""
         Returns a conventional rendering of the character table.
 
+        Arguments:
+            multiplier: (optional) Schur multiplier
+            full: whether the character table for all group elements (True)
+                or one representative per conjugacy class (False, default)
+
         Returns:
 
             A tuple containing a list of strings and an array
 
             - :code:`classes`: a text description of a representative of
-              each conjugacy class as a list
+              each conjugacy class (or each group element) as a list
             - :code:`characters`: a matrix, each row of which lists the
               characters of one irrep
         """
-        # TODO put more effort into nice rendering?
-        classes, idx_repr, _ = self.conjugacy_classes
-        class_sizes = classes.sum(axis=1)
-        representatives = [
-            f"{class_sizes[cls]}x{self[rep]}" for cls, rep in enumerate(idx_repr)
-        ]
-        return representatives, self.character_table_by_class
+        if full:
+            names = [str(g) for g in self]
+            return names, self.character_table(multiplier)
+        else:
+            classes, idx_repr, _ = self.conjugacy_classes
+            class_sizes = classes.sum(axis=1)
+            representatives = [
+                f"{class_sizes[cls]}x{self[rep]}" for cls, rep in enumerate(idx_repr)
+            ]
+            if multiplier is None or np.allclose(multiplier, 1.0, rtol=0, atol=1e-8):
+                # linear representations
+                CT = self.character_table_by_class
+            else:
+                # projective representations
+                CT, _ = self.projective_characters_by_class(multiplier)
+            return representatives, CT
 
     @struct.property_cached
-    def _irrep_matrices(self) -> List[Array]:
+    def _irrep_matrices(self) -> list[Array]:
         # We use Dixon's algorithm (Math. Comp. 24 (1970), 707) to decompose
         # the regular representation of the group into its irreps.
         # We start with a Hermitian matrix E that commutes with every matrix in
@@ -369,14 +599,14 @@ class FiniteGroup(FiniteSemiGroup):
 
         return irreps
 
-    def irrep_matrices(self) -> List[Array]:
+    def irrep_matrices(self) -> list[Array]:
         """
         Returns matrices that realise all irreps of the group.
 
         Returns:
-            A list of 3D arrays such that `self.irrep_matrices()[i][g]` contains
-            the representation of `self[g]` consistent with the characters in
-            `self.character_table()[i]`.
+            A list of 3D arrays such that :code:`self.irrep_matrices()[i][g]`
+            contains the representation of :code:`self[g]` consistent with
+            the characters in :code:`self.character_table()[i]`.
         """
         return self._irrep_matrices
 
