@@ -16,43 +16,106 @@
 # pylint: disable=function-redefined
 
 import itertools
-from typing import Optional
 
 import numpy as np
+from typing import overload, Literal, Any
 
 from netket.utils import HashableArray, struct
 from netket.utils.types import Array, DType, Shape
 from netket.utils.dispatch import dispatch
 
-from ._group import FiniteGroup
-from ._semigroup import Element
+from netket.utils import warn_deprecation, deprecated, deprecated_new_name
+
+from netket.utils.group._group import FiniteGroup
+from netket.utils.group._semigroup import Element
 
 
 class Permutation(Element):
-    def __init__(self, permutation: Array, name: Optional[str] = None):
+    def __init__(
+        self,
+        permutation: Array | None = None,
+        *,  # change one line somewhere
+        name: str | None = None,
+        permutation_array: Array | None = None,
+        inverse_permutation_array: Array | None = None,
+    ):
         r"""
-        Creates a `Permutation` from an array of preimages of :code:`range(N)`.
+        Creates a `Permutation` from either the array of images
+        `permutation_array` or preimages `inverse_permutation_array`.
 
-        Arguments:
-            permutation: 1D array listing :math:`g^{-1}(x)` for all :math:`0\le x < N`
-                (i.e., :code:`V[permutation]` permutes the elements of `V` as desired)
-            name: optional, custom name for the permutation
+        Exactly one argument among `permutation_array` and
+        `inverse_permutation_array` (and the deprecated argument `permutation`)
+        must be specified.
+
+        The deprecated argument `permutation` should be substituted for
+        `inverse_permutation_array`.
+
+        Note that the left action of a permutation on an array `a` is
+        `a[inverse_permutation_array]`.
+
+        Args:
+            permutation: (deprecated) 1D array listing
+                :math:`g^{-1}(x)` for all :math:`0\le x \le N-1`.
+            name: Optional, custom name for the permutation.
+            permutation_array: 1D array listing
+                :math:`g(x)` for all :math:`0\le x \le N-1`.
+            inverse_permutation_array: 1D array listing
+                :math:`g^{-1}(x)` for all :math:`0\le x \le N-1`.
 
         Returns:
-            a `Permutation` object encoding the same permutation
+            A `Permutation` object that encodes the specified permutation.
         """
-        self.permutation = HashableArray(np.asarray(permutation))
+
+        arg_list = [permutation, permutation_array, inverse_permutation_array]
+        if sum([arg is not None for arg in arg_list]) != 1:
+            raise ValueError(
+                "Exactly one argument among `permutation`, `permutation_array` "
+                "and `inverse_permutation_array` must be specified."
+            )
+
+        if permutation is not None:
+            warn_deprecation(
+                "The argument `permutation` is deprecated.\n\n"
+                "In order to clarify notations, you should either pass "
+                "the array of images `permutation_array` or "
+                "preimages `inverse_permutation_array`."
+            )
+            inverse_permutation_array = permutation
+
+        if permutation_array is not None:
+            inverse_permutation_array = np.argsort(permutation_array)
+
+        self._inverse_permutation_array = HashableArray(
+            np.asarray(inverse_permutation_array)
+        )
+
         self.__name = name
 
     def __hash__(self):
-        return hash(self.permutation)
+        return hash(self._inverse_permutation_array)
 
     def __eq__(self, other):
         if isinstance(other, Permutation):
-            return self.permutation == other.permutation
+            return np.array_equal(self.permutation_array, other.permutation_array)
         else:
             return False
 
+    @property
+    def permutation_array(self):
+        return np.asarray(np.argsort(self._inverse_permutation_array))
+
+    @property
+    def inverse_permutation_array(self):
+        return np.asarray(self._inverse_permutation_array)
+
+    @property
+    @deprecated(
+        "Deprecated in favor of `permutation.inverse_permutation_array` or "
+        "`permutation.permutation_array`"
+    )
+    def permutation(self):
+        return self._inverse_permutation_array
+
     @property
     def _name(self):
         return self.__name
@@ -61,25 +124,31 @@ class Permutation(Element):
         if self._name is not None:
             return self._name
         else:
-            return f"Permutation({np.asarray(self).tolist()})"
+            return f"Permutation({self.permutation_array.tolist()})"
 
+    @deprecated_new_name("permutation.inverse_permutation_array")
     def __array__(self, dtype: DType = None):
-        return np.asarray(self.permutation, dtype)
+        return np.asarray(self._inverse_permutation_array, dtype)
 
     def apply_to_id(self, x: Array):
         """Returns the image of indices `x` under the permutation"""
-        return np.argsort(self.permutation)[x]
+        return self.permutation_array[x]
 
 
 @dispatch
 def product(p: Permutation, x: Array):
-    return x[..., p.permutation]
+    # if p.permutation is a HashableArray and x is a jax Array
+    # direct indexing fails, so we call np.asarray on it to extract the
+    # wrapped array
+    # TODO make indexing work with HashableArray directly
+    return x[..., p.inverse_permutation_array]
 
 
 @dispatch
 def product(p: Permutation, q: Permutation):  # noqa: F811
     name = None if p._name is None and q._name is None else f"{p} @ {q}"
-    return Permutation(p(np.asarray(q)), name)
+    inverse_permutation_array = q.inverse_permutation_array[p.inverse_permutation_array]
+    return Permutation(inverse_permutation_array=inverse_permutation_array, name=name)
 
 
 @struct.dataclass
@@ -102,6 +171,11 @@ class PermutationGroup(FiniteGroup):
     def __hash__(self):
         return super().__hash__()
 
+    def __eq__(self, other: "PermutationGroup") -> bool:
+        if isinstance(other, PermutationGroup):
+            return np.array_equal(self.to_array(), other.to_array())
+        return False
+
     def _canonical(self, x: Element) -> Array:
         return x(np.arange(self.degree, dtype=int))
 
@@ -129,7 +203,17 @@ class PermutationGroup(FiniteGroup):
     def __array__(self, dtype=None) -> Array:
         return np.asarray(self.to_array(), dtype=dtype)
 
-    def remove_duplicates(self, *, return_inverse=False) -> "PermutationGroup":
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[False] = False
+    ) -> "PermutationGroup": ...
+
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[True]
+    ) -> tuple["PermutationGroup", Any]: ...
+
+    def remove_duplicates(self, *, return_inverse=False):
         r"""
         Returns a new :code:`PermutationGroup` with duplicate elements (that is,
         elements which represent identical permutations) removed.
@@ -213,6 +297,10 @@ class PermutationGroup(FiniteGroup):
         return self.to_array()[self.inverse][:, x]
 
 
+PermutationGroup.inverse.__doc__ = FiniteGroup.inverse.__doc__
+PermutationGroup.product_table.__doc__ = FiniteGroup.product_table.__doc__
+
+
 @dispatch
 def product(A: PermutationGroup, B: PermutationGroup):  # noqa: F811
     if A.degree != B.degree:
