@@ -18,7 +18,7 @@
 import itertools
 from functools import partial
 from math import pi
-from typing import Dict, Optional, Tuple
+from typing import overload, Literal, Any
 
 import numpy as np
 from scipy.linalg import schur
@@ -28,15 +28,15 @@ from netket.utils.float import comparable, comparable_periodic, is_approx_int
 from netket.utils.types import Array, Shape
 from netket.utils.dispatch import dispatch
 
-from ._group import FiniteGroup
-from ._semigroup import Element, Identity
+from netket.utils.group._group import FiniteGroup
+from netket.utils.group._semigroup import Element, Identity
 
 ############ POINT GROUP SYMMETRY CLASS ########################################
 
 
 @struct.dataclass
 class PGSymmetry(Element):
-    """
+    r"""
     An abstract group element object to geometrically describe point group symmetries.
 
     Construction: `PGSymmetry(W,w)` with an orthogonal matrix `W` and an optional
@@ -54,7 +54,7 @@ class PGSymmetry(Element):
     structure [[W,w],[0,1]].
     """
 
-    def __pre_init__(self, W: Array, w: Optional[Array] = None) -> Tuple[Tuple, Dict]:
+    def __pre_init__(self, W: Array, w: Array | None = None) -> tuple[tuple, dict]:
         W = np.asarray(W)
         if W.ndim != 2 or W.shape[0] != W.shape[1]:
             raise ValueError("W must be a 2D square matrix")
@@ -96,7 +96,7 @@ class PGSymmetry(Element):
     @property
     def is_proper(self) -> bool:
         """Returns True if `self` is a proper rotation (det(`self.matrix`) is +1)."""
-        return np.isclose(np.linalg.det(self.matrix), 1.0)
+        return bool(np.isclose(np.linalg.det(self.matrix), 1.0))
 
     @property
     def is_symmorphic(self) -> bool:
@@ -172,7 +172,7 @@ def _eig(W):
     return np.diag(e), v
 
 
-def _origin_trans(W: Array, w: Array) -> Tuple[Array, Array]:
+def _origin_trans(W: Array, w: Array) -> tuple[Array, Array]:
     """Decomposes a point group symmetry into a pure (improper) rotation around
     an origin and a translation along the axis/plane of the transformation.
     Returns the tuple (origin, translation)."""
@@ -187,7 +187,7 @@ def _origin_trans(W: Array, w: Array) -> Tuple[Array, Array]:
     return origin.real, trans.real
 
 
-def _2D_name(W: Array, w: Optional[Array]) -> str:
+def _2D_name(W: Array, w: Array | None) -> str:
     if W.shape != (2, 2):
         raise ValueError("This function names 2D symmetries")
 
@@ -223,7 +223,7 @@ def _2D_name(W: Array, w: Optional[Array]) -> str:
         raise ValueError("W must be an orthogonal matrix")
 
 
-def _3D_name(W: Array, w: Optional[Array] = None) -> str:
+def _3D_name(W: Array, w: Array | None = None) -> str:
     if W.shape != (3, 3):
         raise ValueError("This function names 3D symmetries")
 
@@ -265,7 +265,6 @@ def _3D_name(W: Array, w: Optional[Array] = None) -> str:
                 return f"Screw({angle}°){_to_rational_vector(trans)}{origin}"
 
     elif _naming_isclose(np.linalg.det(W), -1.0):  # improper rotations
-
         if _naming_allclose(W, -np.eye(3)):  # inversion
             return f"Inv(){origin}"
 
@@ -295,7 +294,7 @@ def _3D_name(W: Array, w: Optional[Array] = None) -> str:
         raise ValueError("W must be an orthogonal matrix")
 
 
-def __to_rational(x: float) -> Tuple[int, int]:
+def __to_rational(x: float) -> tuple[int, int]:
     denom = is_approx_int(x * np.arange(1, 100), atol=_naming_tol)
     if not denom.any():
         raise ValueError
@@ -361,9 +360,9 @@ class PointGroup(FiniteGroup):
     """
     Collection of point group symmetries acting on n-dimensional vectors.
 
-    Group elements need not all be of type :class:`netket.utils.symmetry.PGSymmetry`,
-    only act on such vectors when called. Currently, however, only `Identity` and
-    `PGSymmetry` have canonical forms implemented.
+    Group elements need not all be of type :class:`netket.utils.group.PGSymmetry`,
+    only act on such vectors when called. Currently, however, only
+    :class:`Identity` and :class:`PGSymmetry` have canonical forms implemented.
 
     The class can contain elements that are distinct as objects (e.g.,
     :code:`Identity()` and :code:`Rotation(0)`) but have identical action.
@@ -371,9 +370,9 @@ class PointGroup(FiniteGroup):
     """
 
     ndim: int
-    """Dimensionality of point group operations."""
+    """Dimension of the Euclidean space the point group acts on."""
 
-    unit_cell: Optional[Array] = None
+    unit_cell: Array | None = None
     """Lattice vectors of the unit cell on which the point group acts.
     It is used to match non-symmorphic symmetries modulo lattice translations."""
 
@@ -382,6 +381,8 @@ class PointGroup(FiniteGroup):
 
     @struct.property_cached
     def is_symmorphic(self) -> bool:
+        """:code:`True` if the point group is symmorphic, i.e., if
+        every group element leaves the origin invariant."""
         return all(
             [
                 (True if isinstance(elem, Identity) else elem.is_symmorphic)
@@ -469,7 +470,17 @@ class PointGroup(FiniteGroup):
     def __array__(self, dtype=None) -> Array:
         return np.asarray(self.to_array(), dtype=dtype)
 
-    def remove_duplicates(self, *, return_inverse=False) -> "PointGroup":
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[False] = False
+    ) -> "PointGroup": ...
+
+    @overload
+    def remove_duplicates(
+        self, *, return_inverse: Literal[True]
+    ) -> tuple["PointGroup", Any]: ...
+
+    def remove_duplicates(self, *, return_inverse=False):
         """
         Returns a new :code:`PointGroup` with duplicate elements (that is, elements
         which represent identical transformations) removed.
@@ -515,7 +526,7 @@ class PointGroup(FiniteGroup):
         describe pure point-group transformations around `origin` and `out[i]`
         has the same transformation matrix as `self[i]`.
         """
-        out = []
+        out: list[Element] = []
         for elem in self.elems:
             if isinstance(elem, Identity):
                 out.append(Identity())
@@ -578,6 +589,10 @@ class PointGroup(FiniteGroup):
         return (len(self), self.ndim + 1, self.ndim + 1)
 
 
+PointGroup.inverse.__doc__ = FiniteGroup.inverse.__doc__
+PointGroup.product_table.__doc__ = FiniteGroup.product_table.__doc__
+
+
 def trivial_point_group(ndim: int) -> PointGroup:
     return PointGroup([Identity()], ndim=ndim)
 
