@@ -20,12 +20,11 @@ from .base import (
     expect_and_forces,
 )
 
-from .mc import MCState, MCMixedState, get_local_kernel_arguments, get_local_kernel
-from .exact import ExactState
+# Import types needed for documentation
+from flax.core.scope import CollectionFilter, DenyList  # noqa: F401
 
-# TODO: this is deprecated in favour of netket.experimental.vqs
-# eventually remove this file and import
-from . import experimental
+from .mc import MCState, MCMixedState, get_local_kernel_arguments, get_local_kernel
+from .full_summ import FullSumState
 
 from netket.utils import _hide_submodules
 
