@@ -17,6 +17,7 @@ from netket.utils.dispatch import dispatch
 from netket.operator import (
     AbstractSuperOperator,
     DiscreteOperator,
+    DiscreteJaxOperator,
     Squared,
 )
 
@@ -26,6 +27,13 @@ from .state import MCMixedState
 
 
 # Dispatches to select what expect-kernel to use
+@dispatch
+def get_local_kernel(  # noqa: F811
+    vstate: MCMixedState, Ô: AbstractSuperOperator, chunk_size: int
+):
+    return kernels.local_value_kernel_chunked
+
+
 @dispatch
 def get_local_kernel(  # noqa: F811
     vstate: MCMixedState, Ô: Squared[AbstractSuperOperator], chunk_size: int
@@ -38,3 +46,10 @@ def get_local_kernel(  # noqa: F811
     vstate: MCMixedState, Ô: DiscreteOperator, chunk_size: int
 ):
     return kernels.local_value_op_op_cost_chunked
+
+
+@dispatch
+def get_local_kernel(  # noqa: F811
+    vstate: MCMixedState, Ô: DiscreteJaxOperator, chunk_size: int
+):
+    return kernels.local_value_op_op_cost_chunked
