@@ -12,23 +12,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Callable
+from collections.abc import Callable
 from functools import partial
 
 import jax
 from jax import numpy as jnp
 
-from netket.stats import Stats, statistics as mpi_statistics
+from netket.stats import Stats, statistics
 from netket.utils.types import PyTree
 from netket.utils.dispatch import dispatch
+from netket.utils import HashablePartial
 
 from netket.operator import (
     AbstractOperator,
     DiscreteOperator,
     Squared,
     ContinuousOperator,
+    DiscreteJaxOperator,
 )
-
+from netket.operator._sum import SumGenericOperator
 from netket.vqs.mc import (
     kernels,
     check_hilbert,
@@ -68,18 +70,54 @@ def get_local_kernel(vstate: MCState, Ô: DiscreteOperator):  # noqa: F811
 
 
 @dispatch
-def get_local_kernel_arguments(vstate: MCState, Ô: ContinuousOperator):  # noqa: F811
+def get_local_kernel_arguments(vstate: MCState, Ô: DiscreteJaxOperator):  # noqa: F811
     check_hilbert(vstate.hilbert, Ô.hilbert)
 
     σ = vstate.samples
-    args = Ô._pack_arguments()
-    return σ, args
+    return σ, Ô
+
+
+@dispatch
+def get_local_kernel(vstate: MCState, Ô: DiscreteJaxOperator):  # noqa: F811
+    return kernels.local_value_kernel_jax
+
+
+@dispatch
+def get_local_kernel_arguments(vstate: MCState, Ô: ContinuousOperator):  # noqa: F811
+    check_hilbert(vstate.hilbert, Ô.hilbert)
+    return vstate.samples, Ô
 
 
 @dispatch
 def get_local_kernel(vstate: MCState, Ô: ContinuousOperator):  # noqa: F811
     # TODO: this should be moved other to dispatch in order to support MCMixedState
-    return Ô._expect_kernel
+    return HashablePartial(
+        lambda logpsi, params, x, op: op._expect_kernel(logpsi, params, x)
+    )
+
+
+## sum operators
+
+
+@partial(get_local_kernel_arguments.dispatch, precedence=-2)
+def get_local_kernel_args_sum(vs: MCState, O: SumGenericOperator):
+    samples = vs.samples
+    args = [get_local_kernel_arguments(vs, o)[1] for o in O.operators]
+    return samples, (O.coefficients, args)
+
+
+@partial(get_local_kernel.dispatch, precedence=-2)
+def get_local_kernel_sum(vs: MCState, O: SumGenericOperator):
+    O_kernels = tuple(get_local_kernel(vs, o) for o in O.operators)
+
+    def sum_eval(logpsi, pars, σ, args, O_kernels):
+        coeffs, O_args = args
+        O_locs = 0
+        for O_kernel, O_arg, k in zip(O_kernels, O_args, coeffs):
+            O_locs = O_locs + k * O_kernel(logpsi, pars, σ, O_arg)
+        return O_locs
+
+    return HashablePartial(sum_eval, O_kernels=O_kernels)
 
 
 # Standard implementation of expect for an MCState (pure) and a generic operator
@@ -88,9 +126,10 @@ def get_local_kernel(vstate: MCState, Ô: ContinuousOperator):  # noqa: F811
 # but if somebody wants to override behaviour for an existing operator or define
 # a completely arbitrary novel type of operator, this makes it much easier.
 @dispatch
-def expect(vstate: MCState, Ô: AbstractOperator) -> Stats:  # noqa: F811
+def expect(
+    vstate: MCState, Ô: AbstractOperator, chunk_size: None
+) -> Stats:  # noqa: F811
     σ, args = get_local_kernel_arguments(vstate, Ô)
-
     local_estimator_fun = get_local_kernel(vstate, Ô)
 
     return _expect(
@@ -114,10 +153,9 @@ def _expect(
     σ: jnp.ndarray,
     local_value_args: PyTree,
 ) -> Stats:
-    σ_shape = σ.shape
-
-    if jnp.ndim(σ) != 2:
-        σ = σ.reshape((-1, σ_shape[-1]))
+    n_chains = σ.shape[0]
+    if σ.ndim >= 3:
+        σ = jax.lax.collapse(σ, 0, 2)
 
     def logpsi(w, σ):
         return model_apply_fun({"params": w, **model_state}, σ)
@@ -125,19 +163,7 @@ def _expect(
     def log_pdf(w, σ):
         return machine_pow * model_apply_fun({"params": w, **model_state}, σ).real
 
-    # TODO: Broken until google/jax#11916 is resolved.
-    # should uncomment and remove code below once this is fixed
-    # _, Ō_stats = nkjax.expect(
-    #    log_pdf,
-    #    partial(local_value_kernel, logpsi),
-    #    parameters,
-    #    σ,
-    #    local_value_args,
-    #    n_chains=σ_shape[0],
-    # )
-
     L_σ = local_value_kernel(logpsi, parameters, σ, local_value_args)
-    L_σ = L_σ.reshape((σ_shape[0], -1))
-    Ō_stats = mpi_statistics(L_σ.T)
+    Ō_stats = statistics(L_σ.reshape((n_chains, -1)))
 
     return Ō_stats
